#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <EzGraph.h>
#include "config.h"
#include "draw.h"
#include "obj_draw.h"
#include "obj.h"
#include "stage.h"
#include "emap.h"
#include "layout.h"
#include "file.h"
#include "uty.h"
#include "game.h"

static obj_t objs[OBJ_NUM_MAX];
static obj_t *obj_alive_list[oc_end];
static obj_t *obj_free_list;

obj_t *pop_obj_list(obj_t **pop_list);
void push_obj_list(obj_t *obj, obj_t **push_list);

int check_creature_landed(obj_t *obj);

exclude_result_t exclude_obj_horizontal(obj_t *obj);
exclude_result_t exclude_obj_vertical(obj_t *obj);

static state_req_entry_t req_queue[OBJ_STATE_REQ_MAX];
static int req_num = 0;

#define Size2Id(width, height) (((width)<<4) + (height))

obj_info_t obj_infos[] = {
    {ot_wall, "wall", oc_static, 
     {create_wall, NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_goal, "goal", oc_hard_dynamic, 
     {create_goal, NULL, act_goal, draw_goal, NULL, NULL, NULL, NULL}},
    {ot_stairs_l, "stairs_l", oc_static,
     {create_stairs_l, NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_stairs_r, "stairs_r", oc_static,
     {create_stairs_r, NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_rstairs_l, "rstairs_l", oc_static,
     {create_rstairs_l, NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_rstairs_r, "rstairs_r", oc_static,
     {create_rstairs_r, NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_slope_l, "slope_l", oc_static,
     {create_slope_l, NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_slope_r, "slope_r", oc_static,
     {create_slope_r, NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_sign, "sign", oc_static, 
     {create_sign, NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_sign_hang, "sign_hang", oc_static, 
     {create_sign, NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_sting, "sting", oc_static,    // createot_sting_lʤɤˤʤΤdraw
     {create_sting, NULL, NULL, NULL, NULL, NULL, NULL, NULL}},
    {ot_sting_l, "sting_l", oc_static,  // ʲ4ot_stingȤƺΤcreate
     {NULL, NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_sting_r, "sting_r", oc_static,
     {NULL, NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_sting_u, "sting_u", oc_static,
     {NULL, NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},
    {ot_sting_d, "sting_d", oc_static,
     {NULL, NULL, NULL, draw_static, NULL, NULL, NULL, NULL}},

    {ot_chee, "chee", oc_soft_dynamic,
     {create_chee, setup_creature, act_chee, draw_chee, NULL, 
      NULL, set_motion_chee, set_emap_chee}},


    {ot_fwall, "fwall", oc_hard_dynamic,
     {create_fwall, NULL, act_fwall, draw_motion, NULL, NULL, NULL, NULL}},
    {ot_iwall, "iwall", oc_hard_dynamic,
     {create_iwall, NULL, act_iwall, draw_iwall, NULL, NULL, NULL, NULL}},
    {ot_barrier, "barrier", oc_hard_dynamic,
     {create_barrier, setup_barrier, act_barrier, draw_barrier, NULL,
      NULL, NULL, set_emap_barrier}},
    {ot_pipe, "pipe", oc_hard_dynamic,
	 // 뤷񤭤뤿ᡢdraw_staticpost_drawȤϿ
     {create_pipe, NULL, act_pipe, NULL, draw_static,
      NULL, NULL, NULL}},


    {ot_gondin, "gondin", oc_soft_dynamic,
     {create_gondin, setup_creature, act_creature, draw_creature, NULL, 
	 set_action_gondin, set_motion_gondin, set_emap_gondin}},
    {ot_hatopo, "hatopo", oc_soft_dynamic,
     {create_hatopo, setup_creature, act_creature, draw_creature, NULL, 
      set_action_hatopo, set_motion_hatopo, set_emap_creature}},
    {ot_gagga, "gagga", oc_soft_dynamic,
     {create_gagga, setup_creature, act_creature, draw_gagga, post_draw_gagga,
      set_action_gagga, set_motion_gagga, set_emap_gagga}},

    {ot_nya, "nya", oc_hard_dynamic,
	{create_nya, setup_nya, act_nya, draw_nya, NULL, NULL, NULL, NULL}},
//      set_action_nya, set_motion_nya}},
    {ot_punch, "punch", oc_hard_dynamic,
     {create_punch, NULL, act_punch, draw_punch,  NULL,NULL, NULL, NULL}},
    {ot_yaramasi, "yaramasi", oc_soft_dynamic,
     {create_yaramasi, setup_creature, act_creature, draw_creature,  NULL,
      set_action_yaramasi, set_motion_yaramasi, set_emap_yaramasi}},
    {ot_ojigiri, "ojigiri", oc_soft_dynamic,
     {create_ojigiri, setup_creature, act_creature, NULL, post_draw_ojigiri,
      set_action_ojigiri, set_motion_ojigiri, set_emap_creature}},
    {ot_sue, "sue", oc_soft_dynamic,
     {create_sue, setup_creature, act_creature, draw_creature, NULL,
      set_action_sue, set_motion_sue, set_emap_creature}},
    {ot_chu, "chu", oc_soft_dynamic,
     {create_chu, setup_creature, act_creature, draw_chu, NULL,
      set_action_chu, set_motion_chu, set_emap_creature}},
    {ot_mingo, "mingo", oc_soft_dynamic,
     {create_mingo, setup_creature, act_creature, draw_creature, NULL,
      set_action_mingo, set_motion_mingo, set_emap_mingo}},
    {ot_jumeme, "jumeme", oc_soft_dynamic,
     {create_jumeme, setup_creature, act_creature, draw_jumeme, NULL,
      set_action_jumeme, set_motion_jumeme, set_emap_jumeme}},
    {ot_siguku, "siguku", oc_soft_dynamic,
     {create_siguku, NULL, act_creature, draw_creature, NULL,
      set_action_siguku, set_motion_siguku, set_emap_siguku}},
// Ƥоݤξ夫Τ
// flamedrawpost_drawȤϿ
	{ot_flame, "flame", oc_soft_dynamic,
     {create_flame, NULL, act_flame, NULL, draw_flame,
      NULL, NULL, NULL}},



    {ot_lift, "lift", oc_hard_dynamic,
     {create_lift, setup_lift, act_lift, draw_lift, NULL, NULL, NULL, set_emap_lift}},
    {ot_mswitch, "mswitch", oc_hard_dynamic,
     {create_mswitch, setup_mswitch, act_mswitch, draw_switch, NULL,
      NULL, NULL, NULL}},
    {ot_aswitch, "aswitch", oc_hard_dynamic,
     {create_aswitch, setup_aswitch, act_aswitch, draw_switch, NULL,
      NULL, NULL, set_emap_aswitch}},
    {ot_teleporter, "teleporter", oc_hard_dynamic,
     {create_teleporter, setup_teleporter, act_teleporter, draw_teleporter, 
      NULL, NULL, set_motion_teleporter, set_emap_teleporter}},
    {ot_teleport_dest, "teleport_dest", oc_static,
     {create_teleport_dest, NULL, NULL, NULL,
      NULL, NULL, NULL, NULL}},
    {ot_hub, "hub", oc_static,
     {create_hub, NULL, NULL, NULL,
      NULL, NULL, NULL, NULL}},
};
obj_funcs_t obj_funcs_table[ot_end];
obj_class_t obj_class_table[ot_end];
#define TypeToClass(type) (obj_class_table[type])
char *obj_name_table[ot_end];

creature_images_t creature_image_table[ot_creature_end];
creature_image_info_t creature_image_info_list[] = {
	{ot_chee,
     {
         ii_chee_walk_l0, ii_chee_walk_l1,
         ii_chee_walk_r0, ii_chee_walk_r1, 
         ii_chee_smash_hl1, ii_chee_smash_hr1, 
         ii_chee_smash_hl2, ii_chee_smash_hr2, 
         ii_chee_smash_vl1, ii_chee_smash_vr1, 
         ii_chee_smash_vl2, ii_chee_smash_vr2, 
         ii_chee_blast_l, ii_chee_blast_r,
         ii_chee_pierce_l, ii_chee_pierce_r,
         ii_chee_burn_l, ii_chee_burn_r,
         ii_chee_electrocute_l, ii_chee_electrocute_r,
     }},
	{ot_hatopo,
     {
         ii_hatopo_walk_l0, ii_hatopo_walk_l1,
         ii_hatopo_walk_r0, ii_hatopo_walk_r1, 
         ii_hatopo_smash_hl1, ii_hatopo_smash_hr1, 
         ii_hatopo_smash_hl2, ii_hatopo_smash_hr2, 
         ii_hatopo_smash_vl1, ii_hatopo_smash_vr1, 
         ii_hatopo_smash_vl2, ii_hatopo_smash_vr2, 
         ii_hatopo_blast_l, ii_hatopo_blast_r,
         ii_hatopo_pierce_l, ii_hatopo_pierce_r,
         ii_hatopo_burn_l, ii_hatopo_burn_r,
         ii_hatopo_electrocute_l, ii_hatopo_electrocute_r,
     }},
    {ot_gagga,
     {
         ii_none, ii_none,
         ii_none, ii_none,
         ii_gagga_smash_h1, ii_gagga_smash_h1,
         ii_gagga_smash_h2, ii_gagga_smash_h2,
         ii_gagga_smash_v1, ii_gagga_smash_v1,
         ii_gagga_smash_v2, ii_gagga_smash_v2,
         ii_gagga_blast, ii_gagga_blast,
         ii_gagga_pierce, ii_gagga_pierce,
         ii_gagga_burn, ii_gagga_burn,
         ii_gagga_electrocute, ii_gagga_electrocute,
     }},
	{ot_yaramasi,
     {
         ii_yaramasi_walk_l0, ii_yaramasi_walk_l1,
         ii_yaramasi_walk_r0, ii_yaramasi_walk_r1, 
         ii_yaramasi_smash_hl1, ii_yaramasi_smash_hr1,  
         ii_yaramasi_smash_hl2, ii_yaramasi_smash_hr2, 
         ii_yaramasi_smash_vl1, ii_yaramasi_smash_vr1, 
         ii_yaramasi_smash_vl2, ii_yaramasi_smash_vr2, 
         ii_yaramasi_blast_l, ii_yaramasi_blast_r, 
         ii_yaramasi_pierce_l, ii_yaramasi_pierce_r,
         ii_yaramasi_burn_l, ii_yaramasi_burn_r,
         ii_yaramasi_electrocute_l, ii_yaramasi_electrocute_r,
     }},
	{ot_sue,
     {
         ii_none, ii_none,
         ii_none, ii_none,
         ii_sue_smash_hl1, ii_sue_smash_hr1,
         ii_sue_smash_hl2, ii_sue_smash_hr2,
         ii_sue_smash_vl1, ii_sue_smash_vr1,
         ii_sue_smash_vl2, ii_sue_smash_vr2,
         ii_sue_blast_l, ii_sue_blast_r,
         ii_sue_pierce_l, ii_sue_pierce_r,
         ii_sue_burn_l, ii_sue_burn_r,
         ii_sue_electrocute_l, ii_sue_electrocute_r,
	}},
    {ot_chu,
     {
         ii_chu_walk_l0, ii_chu_walk_l1,
         ii_chu_walk_r0, ii_chu_walk_r1, 
         ii_chu_smash_hl1, ii_chu_smash_hr1,
         ii_chu_smash_hl2, ii_chu_smash_hr2,
         ii_chu_smash_vl1, ii_chu_smash_vr1,
         ii_chu_smash_vl2, ii_chu_smash_vr2,                                  
         ii_chu_blast_l, ii_chu_blast_r,
         ii_chu_pierce_l, ii_chu_pierce_r,
         ii_chu_burn_l, ii_chu_burn_r,  
         ii_chu_electrocute_l, ii_chu_electrocute_r,
     }},
	{ot_mingo,
     {
         // mingoԤ4ޥ˥ʤΤǡset_motionӻꤹ
         ii_none, ii_none,
         ii_none, ii_none,
         ii_mingo_smash_hl1, ii_mingo_smash_hr1, 
         ii_mingo_smash_hl2, ii_mingo_smash_hr2, 
         ii_mingo_smash_vl1, ii_mingo_smash_vr1, 
         ii_mingo_smash_vl2, ii_mingo_smash_vr2, 
         ii_mingo_blast_l, ii_mingo_blast_r,
         ii_mingo_pierce_l, ii_mingo_pierce_r,
         ii_mingo_burn_l, ii_mingo_burn_r,
         ii_mingo_electrocute_l, ii_mingo_electrocute_r,
     }},
	{ot_jumeme,
     {
         ii_jumeme_walk_l0, ii_jumeme_walk_l1,
         ii_jumeme_walk_r0, ii_jumeme_walk_r1, 
         ii_jumeme_smash_hl1, ii_jumeme_smash_hr1, 
         ii_jumeme_smash_hl2, ii_jumeme_smash_hr2, 
         ii_jumeme_smash_vl1, ii_jumeme_smash_vr1, 
         ii_jumeme_smash_vl2, ii_jumeme_smash_vr2, 
         ii_jumeme_blast_l, ii_jumeme_blast_r,
         ii_jumeme_pierce_l, ii_jumeme_pierce_r,
         ii_jumeme_wet_l, ii_jumeme_wet_r,   // 껦˻
         ii_jumeme_electrocute_l, ii_jumeme_electrocute_r,
	 }},
	{ot_siguku,
     {
         ii_none, ii_none,
         ii_none, ii_none,
         ii_siguku_smash_h1, ii_siguku_smash_h1,
         ii_siguku_smash_h2, ii_siguku_smash_h2,
         ii_siguku, ii_siguku, // ľˤ٤ʤ(blockingsplash)
         ii_siguku, ii_siguku,
         ii_siguku_blast_l, ii_siguku_blast_r, // դäȤФ
         ii_siguku, ii_siguku, // ˤϻɤʤ(blockingsplashƾ)
         ii_siguku_mist, ii_siguku_mist, // Ф껦Υѥ
         ii_siguku_mist, ii_siguku_mist, // ХꥢǾäѥ
     }},
	{ot_ojigiri,
     {
         ii_none, ii_none,
         ii_none, ii_none,
         ii_ojigiri_smash_hl1, ii_ojigiri_smash_hr1, 
         ii_ojigiri_smash_hl2, ii_ojigiri_smash_hr2, 
         ii_ojigiri_smash_vl1, ii_ojigiri_smash_vr1, 
         ii_ojigiri_smash_vl2, ii_ojigiri_smash_vr2, 
         ii_ojigiri_l, ii_ojigiri_r,  // !!!
         ii_ojigiri_pierce, ii_ojigiri_pierce,
         ii_ojigiri_burn_l, ii_ojigiri_burn_r,
         ii_ojigiri_electrocute_l, ii_ojigiri_electrocute_r,
     }},
	{ot_gondin,
     {
         ii_none, ii_none,
         ii_none, ii_none,
         ii_gondin_smash_h1, ii_gondin_smash_h1, 
         ii_gondin_smash_h2, ii_gondin_smash_h2, 
         ii_gondin_smash_v1, ii_gondin_smash_v1, 
         ii_gondin_smash_v2, ii_gondin_smash_v2,
         ii_gondin, ii_gondin, // Τޤ޿Ф
         ii_none, ii_none, // ˤϻɤʤ
         ii_gondin_burn, ii_gondin_burn,
         ii_gondin_electrocute, ii_gondin_electrocute,
     }},
};
#define CREATURE_IMAGE_INFO_NUM \
    (sizeof(creature_image_info_list)/sizeof(creature_image_info_t))

// ͭset_motionؿset_motion_creatureǤĤcase
#define SkipDefaultCreatureMotions \
    case os_smash_h1: \
    case os_smash_h2: \
    case os_smash_v1: \
    case os_smash_v2: \
    case os_pierced: \
    case os_burned: \
    case os_electrocuted: \
    case os_digested: \
    case os_bang: \
    case os_blasted: \
        break;


static obj_t *chee = NULL;
static obj_t *goal = NULL;

#define BlastedShockFreezeLeft \
if (obj->type == ot_siguku){ \
    if (check_emap_box_any(Ltx(obj)-1, Lty(obj), 1, obj->height, EM_ICE)){ \
        SetState(obj, os_siguku_freezing); \
    }      \
}          \
else { \
    set_emap_box(Ltx(obj)-4, Lty(obj), 4, obj->height, EM_SHOCK); \
} 
#define BlastedShockFreezeRight \
if (obj->type == ot_siguku){ \
    if (check_emap_box_any(Rtx(obj)+1, Lty(obj), 1, obj->height, EM_ICE)){ \
        SetState(obj, os_siguku_freezing); \
    } \
} \
else { \
    set_emap_box(Rtx(obj)+1, Rty(obj), 4, obj->height, EM_SHOCK); \
}



void obj_init(void){
    int i;
    obj_type_t type;

    for (type = ot_begin ; type < ot_end ; type++){
        obj_name_table[type] = "";
    }
    for (i = 0 ; i < sizeof(obj_infos)/sizeof(obj_info_t) ; i++){
        assert(obj_infos[i].type >= ot_begin);
        assert(obj_infos[i].type < ot_end);
        obj_class_table[obj_infos[i].type] = obj_infos[i].oclass;
        obj_funcs_table[obj_infos[i].type] = obj_infos[i].funcs;
        obj_name_table[obj_infos[i].type] = obj_infos[i].name;
    }
    for (i = 0 ; i < CREATURE_IMAGE_INFO_NUM ; i++){
        creature_image_table[creature_image_info_list[i].type] 
            = creature_image_info_list[i].images;
    }
    setup_obj_list();
}

void setup_obj_list(void){
    int i;

    clear_obj_req_queue();
	// ٤°ʤɤĤʤ褦˥֥ΰꥢƤ
    memset(objs, 0, sizeof(objs));
		for (i = oc_begin ; i < oc_end ; i++){
        obj_alive_list[i] = NULL;
    }
    obj_free_list = objs;
    for (i = 0 ; i < OBJ_NUM_MAX-1 ; i++){
        objs[i].next = &objs[i+1];
        objs[i+1].prev = &objs[i];
    }
    objs[0].prev = objs[OBJ_NUM_MAX-1].next = NULL;
}

obj_t *pop_obj_list(obj_t **pop_list){
    obj_t *obj;

    assert(*pop_list != NULL);
    
    obj = *pop_list;
    *pop_list = (*pop_list)->next;
    (*pop_list)->prev = NULL;
    return obj;
}
void push_obj_list(obj_t *obj, obj_t **push_list){
    obj->prev = NULL;
    obj->next = *push_list;

    if (*push_list != NULL){
        (*push_list)->prev = obj;
    }
    *push_list = obj;
}


obj_t *create_obj_grid(obj_type_t type, int x, int y, dir_t dir){
    return create_obj(type,
                      (x+MAP_GRID_MARGIN)*GRID_SIZE,
                      (y+MAP_GRID_MARGIN)*GRID_SIZE,
                      dir);
}

obj_t *create_obj(obj_type_t type, int x, int y, dir_t dir){
    obj_t *new_obj;
    obj_class_t class;

    assert(obj_free_list != NULL);

    class = TypeToClass(type);

    new_obj = pop_obj_list(&obj_free_list);
    new_obj->class = class;
    new_obj->type = type;
    new_obj->dir = dir;
    new_obj->prev_x = new_obj->x = x;
    new_obj->prev_y = new_obj->y = y;
    new_obj->width = GRID_SIZE;
    new_obj->height = GRID_SIZE;
	new_obj->link = NULL;
    new_obj->attr = 0;
    new_obj->state_stack_top = 0;
	// uեɤͤĤʤ褦ˤ٤0ꥢƤ
	memset(&new_obj->u, 0, sizeof(new_obj->u));
	// !!! 1ե졼ܤpush_horizontalʤΤɤλ
    //	new_obj->u.creature.is_landed = 1;
    SetState(new_obj, os_none);
	if (type != ot_chee){
	    push_obj_list(new_obj, &obj_alive_list[class]);
	}
    if (obj_funcs_table[new_obj->type].create){
        obj_funcs_table[new_obj->type].create(new_obj, x, y, dir);
    }

    return new_obj;
}

obj_t *create_sign_grid(obj_type_t type, int x, int y, int link_no){
    obj_t *new_obj;

	new_obj = create_obj_grid(type, x, y, di_none);
	new_obj->u.sign.no = link_no;
	return new_obj;
}


void delete_obj(obj_t *obj){
    assert(obj != NULL);
    if (obj->prev == NULL){
        obj_alive_list[obj->class] = obj->next;
    }
    else {
        obj->prev->next = obj->next;
    }
    if (obj->next != NULL){
        obj->next->prev = obj->prev;
    }
    
    push_obj_list(obj, &obj_free_list);
}

char *obj_type2name(obj_type_t type){
    assert (type >= ot_begin);
    assert (type < ot_end);
    return obj_name_table[type];
}
obj_type_t obj_name2type(char *name){
    obj_type_t type;

    for (type = ot_begin ; type < ot_end ; type++){
        //        printf(":%s:%s:\n", name, obj_name_table[type]);
        if (strcmp(name, obj_name_table[type]) == 0) return type;
    }
    return ot_none;
}

void forall_obj(int(*f)(obj_t *), obj_class_t class){
    obj_t *obj = obj_alive_list[class];
    obj_t *next;
    int r;
    while (obj){
        r = f(obj);
        //        printf("call func %pfor obj %p ret=%d\n", f, obj, r);
        if (r){ // f0֤ȤϡΥ֥Ȥ
            next = obj->next;
            delete_obj(obj);
            obj = next;
        }
        else {
            obj = obj->next;
        }
    }
}

int act_obj(obj_t *obj){
    if (obj_funcs_table[obj->type].act){
		obj->prev_x = obj->x;
		obj->prev_y = obj->y;
        //		printf("act (%d, %d)->(%d, %d)\n", obj->prev_x, obj->prev_y, obj->x, obj->y);
        return obj_funcs_table[obj->type].act(obj);
    }
    // actؿƤФʤʤ顢֥ȺⵯʤΤ0֤
    return 0;
}


// ೫ϻ˳ƥ֥Ȥemap䥹åξ֤ʤɤꤹ
void setup_stage_obj_list(void){
    int i;
    for (i = 0 ; i < SETUP_TURN ; i++){
		printf("*** setup loop %d ***\n", i);
        forall_obj(setup_stage_obj, oc_hard_dynamic);
        switch_current_emap();
        setup_stage_obj(chee);
        forall_obj(setup_stage_obj, oc_soft_dynamic);
        apply_all_req();
    }
    // ॹȯȤϥꥢ
    SetGameRecord(re_switch_count, 0);
    //    printf("setup_stage_obj_list\n");
    //    exit(1);
}

int setup_stage_obj(obj_t *obj){
    //    printf("call setup func %p. for obj type %d\n", obj_funcs_table[obj->type].setup, obj->type);
    if (obj_funcs_table[obj->type].setup){
        //        printf("call setup func.\n");
        obj_funcs_table[obj->type].setup(obj);
    }
    return 0;
}


void draw_obj_list(void){
    forall_obj(draw_obj, oc_static);
    forall_obj(draw_obj, oc_hard_dynamic);
    forall_obj(draw_obj, oc_soft_dynamic);
	draw_chee(chee);
    forall_obj(post_draw_obj, oc_hard_dynamic);
    forall_obj(post_draw_obj, oc_soft_dynamic);
}
int draw_obj(obj_t *obj){
    if (obj_funcs_table[obj->type].draw){
		obj_funcs_table[obj->type].draw(obj);
	}
    return 0;
}
int post_draw_obj(obj_t *obj){
    if (obj_funcs_table[obj->type].post_draw){
		obj_funcs_table[obj->type].post_draw(obj);
	}
    return 0;
}

void clear_obj_req_queue(void){
    req_num = 0;
}

void push_obj_req(obj_t *dest, obj_t *src, obj_state_t state){
    assert(req_num < OBJ_STATE_REQ_MAX);
    req_queue[req_num].dest = dest;
    req_queue[req_num].src = src;
    req_queue[req_num].state = state;
    req_num ++;
}


void apply_req(obj_t *dest, obj_t *src, obj_state_t state){
    printf("apply req to state %d (%p->%p)\n", state, src, dest);
    // ̤³ʤʤˤ⤷ʤ
    // 衢̤³֤٤ǤϤʤ
    // ơեκߥӾǵꤦ
    // ξ硢å夵ΤϹޤʤΤǡ
    // ؤȵڤԤʤ
    // ʤơκݤΥ󥯲˷ٹåϽФƤ
    if (dest == NULL) return;
    // оݤϥ֤ʤ餽Ŭ
    if (dest->type == ot_hub){
        apply_req_to_hub(dest, src, state);
    }
	// ῩоݤǤῩѤߤʤ饭󥻥
    else if (state == os_eaten && !IsAlive(dest)){
        SetState(src, os_cancel);
    }
    else if (state == os_gondin_eaten && !IsAlive(dest)){
        SetState(src, os_cancel);
    }
    else if (state == os_req_on && PrevState(dest) == os_on){
        // os_req_on -> os_on1󤫤뤬
        //   Ȥos_on -> os_req_on -> os_onȤʤ
        //   req_onǤäOFFˤʤäƤޤ
        //   Τᡢos_onos_req_onڤ괹褦ȤȤ
        //   os_onΤޤޤˤ
        // PrevStateӤ虜虜SetStateΤϡ
        //   Ʊ쥿os_req_offƤޤäƤ˥󥻥뤹뤿
        //        printf("set directly os_on.\n");
        SetState(dest, os_on);
    }
    else if (state == os_req_off && PrevState(dest) == os_off){
        // Ʊ
        //        printf("set directly os_off.\n");
        SetState(dest, os_off);
    }
    else{
        SetState(dest, state);
    }
}

void apply_all_req(void){
    obj_t *src, *dest;
    obj_state_t state;
    while (req_num > 0){
        req_num --;
        dest = req_queue[req_num].dest;
        src = req_queue[req_num].src;
        state = req_queue[req_num].state;
		apply_req(dest, src, state);
	}
}



void create_chee(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = CHEE_SIZE;
    obj->height = CHEE_SIZE;
	SetAttr(obj, at_meat);
	SetAttr(obj, at_can_climb);
    obj->u.creature.move_x = 0;
    obj->dir = dir;
    obj->motion = (dir == di_left) ? ii_chee_walk_l0 : ii_chee_walk_r0;
	chee = obj;
    obj->u.creature.land_ofst = 1;
}
int setup_chee(obj_t *obj){
    if (check_creature_landed(obj)){
        SetLanded(obj, 1);
    }
    set_emap_chee(obj);
    return 0;
}

void set_chee(obj_t *obj){
	chee = obj;
}

obj_t *get_chee(void){
    return chee;
}

void create_gondin(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = GONDIN_SIZE;
    obj->height = GONDIN_SIZE;
	SetAttr(obj, at_vegetable);
	SetAttr(obj, at_immune_pierce);
	SetAttr(obj, at_immune_water);
	obj->motion = ii_gondin;
}

int setup_gondin(obj_t *obj){
    if (check_creature_landed(obj)){
        puts("gondin landed.");
        SetLanded(obj, 1);
    }
    set_emap_gondin(obj);
    return 0;
}

void create_hatopo(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = HATOPO_SIZE;
    obj->height = HATOPO_SIZE;
	SetAttr(obj, at_meat);
	SetAttr(obj, at_can_climb);
	SetAttr(obj, at_immune_water);
    obj->dir = dir;
    obj->u.creature.move_x = (dir == di_left) ? -1 : 1;
    obj->motion = (dir == di_left) ? ii_hatopo_walk_l0 : ii_hatopo_walk_r0;
    //	obj->u.creature.is_walking = 1; // ľȿžʤ褦ˤ뤿
    obj->u.creature.land_ofst = 1;
}

void create_yaramasi(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = YARAMASI_WIDTH;
    obj->height = YARAMASI_HEIGHT;
	SetAttr(obj, at_meat);
    //	SetAttr(obj, at_can_climb);
	SetAttr(obj, at_immune_water);
	SetAttr(obj, at_immune_pierce);
    obj->dir = dir;
    obj->u.creature.move_x = (dir == di_left) ? -1 : 1;
    obj->motion = (dir == di_left) ? ii_yaramasi_walk_l0 : ii_yaramasi_walk_r0;
    //	obj->u.creature.is_walking = 1; // ľȿžʤ褦ˤ뤿
}

void create_gagga(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = GAGGA_SIZE;
    obj->height = GAGGA_SIZE;
	SetAttr(obj, at_meat);
	SetAttr(obj, at_immune_water);
    obj->motion = ii_gagga_init;
	obj->dir = di_left;
}
void create_ojigiri(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = OJIGIRI_WIDTH;
    obj->height = OJIGIRI_HEIGHT;
	SetAttr(obj, at_vegetable);
	SetAttr(obj, at_rooted);
	SetAttr(obj, at_immune_water);
    obj->dir = dir;
    obj->motion = (dir == di_left) ? ii_ojigiri_l : ii_ojigiri_r;
}
void create_sue(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = SUE_WIDTH;
    obj->height = SUE_HEIGHT;
	SetAttr(obj, at_meat);
	SetAttr(obj, at_immune_flame);
	SetAttr(obj, at_immune_water);
    obj->dir = dir;
    obj->motion = (dir == di_left) ? ii_sue_high_close_l : ii_sue_high_close_r;
	SetState(obj, os_head_high);
}
void create_chu(obj_t *obj, int ox, int oy, dir_t dir){
	// CHU_WALK_HEIGHT==3create_obj_grid4ñ̤ʤΤ
	// Ĳˤ餷Ƥ
	obj->y ++;
    obj->width = CHU_WIDTH;
    obj->height = CHU_WALK_HEIGHT;
	SetAttr(obj, at_meat);
	SetAttr(obj, at_can_climb);
	SetAttr(obj, at_immune_water);
    obj->dir = dir;
    obj->motion = (dir == di_left) ? ii_chu_walk_l0 : ii_chu_walk_r0;
    obj->u.creature.move_x = (dir == di_left) ? -1 : 1;
    //	obj->u.creature.is_landed = 1; // ľȿžʤ褦ˤ뤿
    //	obj->u.creature.is_walking = 1; // ľȿžʤ褦ˤ뤿
    //    obj->u.creature.land_ofst = 1;
}
void create_mingo(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = MINGO_SIZE;
    obj->height = MINGO_SIZE;
	SetAttr(obj, at_meat);
	SetAttr(obj, at_can_climb);
	SetAttr(obj, at_immune_water);
	SetAttr(obj, at_immune_pierce); // Фդϻ̵Τ̽
    obj->dir = dir;
    obj->u.creature.move_x = (dir == di_left) ? -1 : 1;
    obj->motion = (dir == di_left) ? ii_mingo_walk_l0 : ii_mingo_walk_r0;
    //	obj->u.creature.is_walking = 1; // ľȿžʤ褦ˤ뤿
    obj->u.creature.land_ofst = 1;
}
void create_jumeme(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = JUMEME_WIDTH;
    obj->height = JUMEME_HEIGHT;
	SetAttr(obj, at_meat);
	SetAttr(obj, at_can_climb);
	SetAttr(obj, at_immune_flame);
    obj->dir = dir;
    obj->u.creature.move_x = (dir == di_left) ? -1 : 1;
    obj->motion = (dir == di_left) ? ii_jumeme_walk_l0 : ii_jumeme_walk_r0;
    //	obj->u.creature.is_walking = 1; // ľȿžʤ褦ˤ뤿
    obj->u.creature.land_ofst = 1;
}

void create_siguku(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = SIGUKU_WIDTH;
    obj->height = SIGUKU_HEIGHT;
//	SetAttr(obj, at_item);
	SetAttr(obj, at_immune_water);
	SetAttr(obj, at_turn_to_mist);
    obj->motion = ii_siguku;
}

void create_flame(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = FLAME_WIDTH;
    obj->height = FLAME_HEIGHT;
	SetAttr(obj, at_immune_flame);
	SetAttr(obj, at_turn_to_mist);
    obj->dir = dir;
    obj->motion = (dir == di_left) ? ii_flame_l : ii_flame_r;
	// Υǽʤ褦ˡиflame̤
	set_emap_flame(obj);
}
void create_nya(obj_t *obj, int ox, int oy, dir_t dir){
	int px, ex;
	obj->width = NYA_WIDTH;
    obj->height = NYA_HEIGHT;
    obj->dir = dir;
	if (dir == di_left){
		obj->motion = ii_nya_on_l;
        //		px = ox-4;
		px = PunchInitPosLeft(obj);
		ex = ox+obj->width-1-3;
	}
	else {
		obj->motion = ii_nya_on_r;
        //		px = ox+8;
		px = PunchInitPosRight(obj);
		ex = ox;
	}
	obj->link = create_obj(ot_punch, px, oy+8+1, dir);
	obj->link->link = obj;
	// Τblock°Ĥ
    set_emap_static_blockbox(ox, oy+2, obj->width, obj->height-2);
	set_emap_static_blockbox(ex, oy, 3, 2);
}
void create_punch(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = PUNCH_WIDTH;
    obj->height = PUNCH_HEIGHT;
    obj->dir = dir;
	obj->motion = (dir == di_left) ? ii_punch_l : ii_punch_r;
}

void create_wall(obj_t *obj, int ox, int oy, dir_t dir){
    set_emap_static_blockbox(ox, oy, obj->width, obj->height);
}

void create_goal(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = GOAL_WIDTH;
    obj->height = GOAL_HEIGHT;
	obj->motion = ii_goal_closed;
	goal = obj;
}

void create_sign(obj_t *obj, int ox, int oy, dir_t dir){
	obj->width = SIGN_WIDTH;
	obj->height = SIGN_HEIGHT;
	set_emap_owner_static_box(obj->x, obj->y, obj->width, obj->height,
                              obj, en_sign);
}

void create_sting(obj_t *obj, int ox, int oy, dir_t dir){
    int x, y;
    switch (dir){
    case di_left:
        obj->type = ot_sting_l;
        for (x = 0 ; x < obj->width ; x++){
            SetEMapStatic(ox+x, oy+obj->height/2-1, EM_PIERCE);
            SetEMapStatic(ox+x, oy+obj->height/2, EM_PIERCE);
            SetEMapStatic(ox+x, oy+obj->height/2-1, EM_BLOCK_RIGHT);
            SetEMapStatic(ox+x, oy+obj->height/2, EM_BLOCK_RIGHT);
        }
        set_emap_static_box(ox+obj->width, oy+1, 1, obj->height-2, 
                            EM_BLOCK_LEFT);
        set_emap_static_box(ox+obj->width-1, oy+1, 1, obj->height-2,
                            EM_BLOCK_UP);
        set_emap_static_box(ox+obj->width-1, oy+1, 1, obj->height-2,
                            EM_BLOCK_DOWN);
        break;
    case di_right:
        obj->type = ot_sting_r;
        for (x = 0 ; x < obj->width ; x++){
            SetEMapStatic(ox+x, oy+obj->height/2-1, EM_PIERCE);
            SetEMapStatic(ox+x, oy+obj->height/2, EM_PIERCE);
            SetEMapStatic(ox+x, oy+obj->height/2-1, EM_BLOCK_LEFT);
            SetEMapStatic(ox+x, oy+obj->height/2, EM_BLOCK_LEFT);
        }
        set_emap_static_box(ox-1, oy+1, 1, obj->height-2, EM_BLOCK_RIGHT);
        set_emap_static_box(ox, oy+1, 1, obj->height-2, EM_BLOCK_UP);
        set_emap_static_box(ox, oy+1, 1, obj->height-2, EM_BLOCK_DOWN);
        break;
    case di_up:
        obj->type = ot_sting_u;
        for (y = 0 ; y < obj->height ; y++){
            SetEMapStatic(ox+obj->width/2-1, oy+y, EM_PIERCE);         
            SetEMapStatic(ox+obj->width/2, oy+y, EM_PIERCE);         
            SetEMapStatic(ox+obj->width/2-1, oy+y, EM_BLOCK_DOWN);
            SetEMapStatic(ox+obj->width/2, oy+y, EM_BLOCK_DOWN);         
        }
        set_emap_static_box(ox+1, oy+obj->height, obj->width-2, 1,
                            EM_BLOCK_UP);
        set_emap_static_box(ox+1, oy+obj->height-1, obj->width-2, 1,
                            EM_BLOCK_LEFT);
        set_emap_static_box(ox+1, oy+obj->height-1, obj->width-2, 1,
                            EM_BLOCK_RIGHT);
        break;
    case di_down:
        obj->type = ot_sting_d;
        for (y = 0 ; y < obj->height ; y++){
            SetEMapStatic(ox+obj->width/2-1, oy+y, EM_PIERCE);
            SetEMapStatic(ox+obj->width/2, oy+y, EM_PIERCE);
            SetEMapStatic(ox+obj->width/2-1, oy+y, EM_BLOCK_UP);
            SetEMapStatic(ox+obj->width/2, oy+y, EM_BLOCK_UP);
        }
        set_emap_static_box(ox+1, oy-1, obj->width-2, 1, EM_BLOCK_DOWN);
        set_emap_static_box(ox+1, oy, obj->width-2, 1, EM_BLOCK_LEFT);
        set_emap_static_box(ox+1, oy, obj->width-2, 1, EM_BLOCK_RIGHT);
        break;
    default:
        assert(0);
        break;
    }
    obj->dir = dir;
}
void create_fwall(obj_t *obj, int ox, int oy, dir_t dir){
    set_emap_static_blockbox(ox, oy, obj->width, obj->height);
    set_emap_static_box(ox, oy, obj->width, obj->height, EM_FWALL);
	obj->motion = ii_fwall;
}
void create_iwall(obj_t *obj, int ox, int oy, dir_t dir){
    set_emap_static_blockbox(ox, oy, obj->width, obj->height);
    set_emap_static_box(ox, oy, obj->width, obj->height, EM_ICE);
	obj->motion = ii_iwall;
}
void create_barrier(obj_t *obj, int ox, int oy, dir_t dir){
    SetState(obj, os_on);
    SavePrevState(obj);
	obj->motion = ii_barrier0;
    set_emap_blockbox(ox, oy, obj->width, obj->height);
}
void create_pipe(obj_t *obj, int ox, int oy, dir_t dir){
    set_emap_static_blockbox(ox, oy, obj->width, obj->height);
	obj->motion = ii_pipe;
    SetState(obj, os_on);
}
void create_slope_l(obj_t *obj, int ox, int oy, dir_t dir){
    int x, y;

    for (y = 0 ; y < obj->height ; y++){
        x = obj->width-y-1;
        // ؤοɻ
        SetEMapStatic(ox+x, oy+y-1, EM_BLOCK_DOWN); 
        SetEMapStatic(ox+x-1, oy+y, EM_BLOCK_RIGHT); 
        // ̸
        SetEMapStatic(ox+x, oy+y-1, EM_PUSH_LEFTDOWN);
    }
    // ̤Ǥʤդοɻ
    for (y = 0 ; y < obj->height ; y++){
        SetEMapStatic(ox+obj->width, oy+y, EM_BLOCK_LEFT);
        SetEMapStatic(ox+obj->width-1, oy+y, EM_EXCLUDE_RIGHT); 
    }
    for (x = 0 ; x < obj->width ; x++){
        SetEMapStatic(ox+x, oy+obj->height, EM_BLOCK_UP); 
        SetEMapStatic(ox+x, oy+obj->height-1, EM_EXCLUDE_DOWN); 
    }
    // פʤɤǤн
    for (y = 0 ; y < obj->height ; y++){
        for (x = obj->width-y-1 ; x < obj->width ; x++){
            SetEMapStatic(ox+x, oy+y, EM_EXCLUDE_LEFT); 
        }
    }
}
void create_slope_r(obj_t *obj, int ox, int oy, dir_t dir){
    int x, y;
    for (y = 0 ; y < obj->height ; y++){
        x = y;
        // ؤοɻ
        SetEMapStatic(ox+x, oy+y-1, EM_BLOCK_DOWN); 
        SetEMapStatic(ox+x+1, oy+y, EM_BLOCK_LEFT); 
        // ̸
        SetEMapStatic(ox+x, oy+y-1, EM_PUSH_RIGHTDOWN);
    }
    // ̤Ǥʤդοɻ
    for (y = 0 ; y < obj->height ; y++){
        SetEMapStatic(ox-1, oy+y, EM_BLOCK_RIGHT); 
        SetEMapStatic(ox, oy+y, EM_EXCLUDE_LEFT); 
    }
    for (x = 0 ; x < obj->width ; x++){
        SetEMapStatic(ox+x, oy+obj->height, EM_BLOCK_UP); 
        SetEMapStatic(ox+x, oy+obj->height-1, EM_EXCLUDE_DOWN); 
    }
    // פʤɤǤн
    for (y = 0 ; y < obj->height ; y++){
        for (x = 0 ; x < y+1 ; x++){
            SetEMapStatic(ox+x, oy+y, EM_EXCLUDE_RIGHT); 
        }
    }
}

void create_stairs_l(obj_t *obj, int ox, int oy, dir_t dir){
    int x, y;
    for (y = 0 ; y < obj->height ; y++){
        for (x = obj->width-y-1 ; x < obj->width ; x++){
            SetEMapStatic(ox+x, oy+y-1, EM_BLOCK_DOWN); 
            SetEMapStatic(ox+x-1, oy+y, EM_BLOCK_RIGHT); 
            // @note EXCLUDE_UPĤȿʿեȤȳʤ˶ޤƤ
            // ʤ˾زФ
            // ˤˤʤΤ
            // 줬ʤȿľեȤȳʤ˶ޤ줿Ȥ٤Ƚ꤬Τ
            // ͤȤ
            // ʤľեȤȤδ֤˶ޤ줿ȤϲФ줺٤
            SetEMapStatic(ox+x, oy+y, EM_EXCLUDE_LEFT | EM_EXCLUDE_UP);
			// ĤƤʤaswitchνŤȽ˼Ԥ
			// 줬ȳʤαʢä֤Ǥ֤鲼
//			SetEMapStatic(ox+x, oy+y, eb_exclude_up); 			  
        }
    }
    // ̤Ǥʤդοɻ
    for (y = 0 ; y < obj->height ; y++){
        SetEMapStatic(ox+obj->width, oy+y, EM_BLOCK_LEFT); 
        SetEMapStatic(ox+obj->width-1, oy+y, EM_EXCLUDE_RIGHT); 
    }
    for (x = 0 ; x < obj->width ; x++){
        SetEMapStatic(ox+x, oy+obj->height, EM_BLOCK_UP); 
        SetEMapStatic(ox+x, oy+obj->height-1, EM_EXCLUDE_DOWN); 
    }
    SetEMapStatic(ox, oy+obj->height-1, EM_EXCLUDE_LEFT); 
}
void create_stairs_r(obj_t *obj, int ox, int oy, dir_t dir){
    int x, y;
    for (y = 0 ; y < obj->height ; y++){
        for (x = 0 ; x <= y ; x++){
            SetEMapStatic(ox+x, oy+y-1, EM_BLOCK_DOWN); 
            SetEMapStatic(ox+x+1, oy+y, EM_BLOCK_LEFT); 
            SetEMapStatic(ox+x, oy+y, EM_EXCLUDE_RIGHT | EM_EXCLUDE_UP); 
        }
    }
    // ̤Ǥʤդοɻ
    for (y = 0 ; y < obj->height ; y++){
        SetEMapStatic(ox-1, oy+y, EM_BLOCK_RIGHT); 
        SetEMapStatic(ox, oy+y, EM_EXCLUDE_LEFT); 
    }
    for (x = 0 ; x < obj->width ; x++){
        SetEMapStatic(ox+x, oy+obj->height, EM_BLOCK_UP); 
        SetEMapStatic(ox+x, oy+obj->height-1, EM_EXCLUDE_DOWN); 
    }
    SetEMapStatic(ox+obj->width-1, oy+obj->height-1, EM_EXCLUDE_RIGHT); 
}
void create_rstairs_l(obj_t *obj, int ox, int oy, dir_t dir){
    int x, y;
	for (y = 0 ; y < obj->height ; y++){
        for (x = y ; x < obj->width ; x++){
            SetEMapStatic(ox+x, oy+y+1, EM_BLOCK_UP); 
            SetEMapStatic(ox+x-1, oy+y, EM_BLOCK_RIGHT); 
            // EXCLUDE_DOWNĤȡ֤ΤΤΡפü
            // ޤäƻ̡
            //if (y > 0) SetEMapStatic(ox+x, oy+y-1, EM_EXCLUDE_DOWN); 
            //SetEMapStatic(ox+x, oy+y, EM_EXCLUDE_LEFT | EM_EXCLUDE_DOWN); 
            SetEMapStatic(ox+x, oy+y, EM_EXCLUDE_LEFT); 
        }
    }
    // ̤Ǥʤդοɻ
    for (y = 0 ; y < obj->height ; y++){
        SetEMapStatic(ox+obj->width, oy+y, EM_BLOCK_LEFT); 
        SetEMapStatic(ox+obj->width-1, oy+y, EM_EXCLUDE_RIGHT); 
    }
    for (x = 0 ; x < obj->width ; x++){
        SetEMapStatic(ox+x, oy-1, EM_BLOCK_DOWN); 
        SetEMapStatic(ox+x, oy, EM_EXCLUDE_UP); 
    }
    SetEMapStatic(ox, oy, EM_EXCLUDE_LEFT); 
}
void create_rstairs_r(obj_t *obj, int ox, int oy, dir_t dir){
    int x, y;
    for (y = 0 ; y < obj->height ; y++){
        for (x = 0 ; x <= obj->width-y-1 ; x++){
            SetEMapStatic(ox+x, oy+y+1, EM_BLOCK_UP); 
            SetEMapStatic(ox+x+1, oy+y, EM_BLOCK_LEFT); 
            //            SetEMapStatic(ox+x, oy+y, EM_EXCLUDE_RIGHT | EM_EXCLUDE_DOWN); 
            //            if (y > 0) SetEMapStatic(ox+x, oy+y-1, EM_EXCLUDE_DOWN); 
            SetEMapStatic(ox+x, oy+y, EM_EXCLUDE_RIGHT); 
        }
    }
    // ̤Ǥʤդοɻ
    for (y = 0 ; y < obj->height ; y++){
        SetEMapStatic(ox-1, oy+y, EM_BLOCK_RIGHT); 
        SetEMapStatic(ox, oy+y, EM_EXCLUDE_LEFT); 
    }
    for (x = 0 ; x < obj->width ; x++){
        SetEMapStatic(ox+x, oy-1, EM_BLOCK_DOWN); 
        SetEMapStatic(ox+x, oy, EM_EXCLUDE_UP); 
    }
    SetEMapStatic(ox+obj->width-1, oy, EM_EXCLUDE_RIGHT); 
}
void create_lift(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = LIFT_DEFAULT_WIDTH;
    obj->dir = dir;
	SetState(obj, os_on);
    set_emap_blockbox(ox, oy, obj->width, obj->height);
}

void create_mswitch(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = SWITCH_DEFAULT_WIDTH;
    set_emap_static_blockbox(ox, oy, obj->width, obj->height);
	SetState(obj, os_off);
    SavePrevState(obj);
    obj->motion = ii_mswitch_off;
}

void create_aswitch(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = SWITCH_DEFAULT_WIDTH;
    //    aswitchϹ⤵ѤΤǡstaticemapϤʤ
    //    set_emap_static_blockbox(ox, oy, obj->width, obj->height);
    //    SetState(obj, os_nonweighted_off);
    SetState(obj, os_off);
    SavePrevState(obj);
    obj->motion = ii_aswitch_off;
}
void create_teleporter(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = TELEPORTER_DEFAULT_WIDTH;
    set_emap_static_blockbox(ox, oy, obj->width, obj->height);
    SetState(obj, os_on);
    obj->motion = ii_teleporter_on;
}
void create_teleport_dest(obj_t *obj, int ox, int oy, dir_t dir){
}

void create_hub(obj_t *obj, int ox, int oy, dir_t dir){
	obj->u.hub.in_num = 0;
	obj->u.hub.out_num = 0;
    obj->u.hub.in_flags = 0;
	obj->u.hub.logic = lo_and;  // ǥեȤǤAND
    obj->u.hub.is_reverse = 0;
}
void set_hub_logic(obj_t *obj, logic_t logic){
	obj->u.hub.logic = logic;
}
void set_hub_reverse(obj_t *obj, int is_reverse){
    obj->u.hub.is_reverse = is_reverse;
}

void register_hub_in(obj_t *hub, obj_t *obj){
    if (hub->u.hub.in_num == HUB_IN_MAX){
        log_error("Too many input to hub (max:%d).\n", HUB_IN_MAX);
        exit(1);
    }
    hub->u.hub.in_objs[hub->u.hub.in_num] = obj;
    hub->u.hub.in_num ++;
    printf("register hub in %p(%d, %d)<-%p(%d, %d) in_num:->%d\n", hub, hub->x/GRID_SIZE, hub->y/GRID_SIZE, obj, obj->x/GRID_SIZE, obj->y/GRID_SIZE, hub->u.hub.in_num);
}

void register_hub_out(obj_t *hub, obj_t *obj){
    if (hub->u.hub.out_num == HUB_OUT_MAX){
        log_error("Too many output to hub (max:%d).\n", HUB_OUT_MAX);
        exit(1);
    }
    hub->u.hub.out_objs[hub->u.hub.out_num] = obj;
    hub->u.hub.out_num ++;
    printf("register hub out %p(%d, %d)->%p(%d, %d) in_num:->%d\n", hub, hub->x/GRID_SIZE, hub->y/GRID_SIZE, obj, obj->x/GRID_SIZE, obj->y/GRID_SIZE, hub->u.hub.out_num);
}

void apply_req_to_hub(obj_t *hub, obj_t *src, obj_state_t state){
    int i;
    int in_ix = -1;
    onoff_t onoff = oo_invalid;
    obj_state_t req;

    printf("apply req to hub(%d, %d) in_num:%d\n", hub->x, hub->y, hub->u.hub.in_num);
    for (i = 0 ; i < hub->u.hub.in_num ; i++){
        printf("src:%p in_objs[%d]=%p(%d,%d)\n", src, i, hub->u.hub.in_objs[i],
               hub->u.hub.in_objs[i]->x/GRID_SIZE, hub->u.hub.in_objs[i]->y/GRID_SIZE);
        if (src == hub->u.hub.in_objs[i]){
            in_ix = i;
            break;
        }
    }
    printf("(%d, %d) in_num:%d src type:%d ms:%d as:%d bar:%d\n", hub->x/GRID_SIZE, hub->y/GRID_SIZE, hub->u.hub.in_num, src->type, ot_mswitch, ot_aswitch, ot_barrier);
    assert(in_ix >= 0);

    if (state == os_req_on || state == os_on){
        printf("hub req on:\n");
        if (CheckHubInFlag(hub, in_ix) == 0){
            printf("  flag on\n");
            hub->u.hub.on_num ++;
            SetHubInFlag(hub, in_ix);
        }
    }
    else if (state == os_req_off || state == os_off){
        printf("hub req off:\n");
        if (CheckHubInFlag(hub, in_ix) != 0){
            printf("  flag off\n");
            hub->u.hub.on_num --;
            ResetHubInFlag(hub, in_ix);
        }
    }
    /*
    hub->u.hub.on_num = 0;
    for (i = 0 ; i < hub->u.hub.in_num ; i++){
        hub->u.hub.on_num += (CheckHubInFlag(hub, i) ? 1 : 0);
    }
    */
    printf("hub count %d/%d %x\n", hub->u.hub.on_num, hub->u.hub.in_num, hub->u.hub.in_flags);
	switch (hub->u.hub.logic){
	case lo_and:
        puts("and");
	    onoff = (hub->u.hub.on_num == hub->u.hub.in_num) ? oo_on : oo_off;
		break;
	case lo_or:
        puts("ro");
	    onoff = (hub->u.hub.on_num > 0) ? oo_on : oo_off;
		break;
	case lo_xor:
	    onoff = (hub->u.hub.on_num % 2 == 1) ? oo_on : oo_off;
        printf("xor onoff:%d\n", onoff);
		break;
	case lo_majority:
	    onoff = (hub->u.hub.on_num*2 >= hub->u.hub.in_num) ? oo_on : oo_off;
        printf("maj onoff:%d %d %d\n", onoff, hub->u.hub.on_num, hub->u.hub.in_num);
		break;
	default:
		assert(0);
		break;
	}
    assert(onoff != oo_invalid);
    if (hub->u.hub.is_reverse){
        req = (onoff == oo_on) ? os_req_off : os_req_on;
    }
    else {
        req = (onoff == oo_on) ? os_req_on : os_req_off;
    }
    printf("hub (%d, %d) %d out. OUTPUT:%s(%d)\n",
           hub->x, hub->y,
           hub->u.hub.out_num,
           req==os_req_on ? "req_on" : "req_off", req);
    for (i = 0 ; i < hub->u.hub.out_num ; i++){
        // HUBФHUBsrcˤƺƵŬ
        apply_req(hub->u.hub.out_objs[i], hub, req);
    }
    printf("hub (%d, %d) apply end.\n",
           hub->x, hub->y);
}

void print_hub(obj_t *hub){
    assert(hub->type == ot_hub);
    printf("HUB(%d, %d) %d in(s) %d out(s)\n",
           hub->x, hub->y, hub->u.hub.in_num, hub->u.hub.out_num);
}

void set_grid_width_static(obj_t *obj, int width){
    obj->width = width * GRID_SIZE;
    set_emap_static_blockbox(obj->x, obj->y, obj->width, obj->height);
	// !!! emapνȺޤǡŪdynamic emapˤ񤤤Ƥ
    set_emap_blockbox(obj->x, obj->y, obj->width, obj->height);
}

void set_grid_width_dynamic(obj_t *obj, int width){
    obj->width = width * GRID_SIZE;
	// ȤꤢĹΤΤblock°Ȳ
	// liftactꤹ뤫餿֤פϤʤϤ
    set_emap_blockbox(obj->x, obj->y, obj->width, obj->height);
}
void set_grid_height_dynamic(obj_t *obj, int height){
    obj->height = height * GRID_SIZE;
	// ȤꤢĹΤΤblock°Ȳ
	// lift, barrieractꤹ뤫餿֤פϤʤϤ
    set_emap_blockbox(obj->x, obj->y, obj->width, obj->height);
}

int check_creature_landed(obj_t *obj){
    int x;

//    puts("landed?");
    //    for (i = 0 ; i < obj->width ; i++){
    for (x = LandLx(obj) ; x <= LandRx(obj) ; x++)
    {
        if (CheckEMap(x, Lby(obj), EM_BLOCK_DOWN)) return 1;
    }
//    puts("landed no");
    return 0;
}

int check_creature_block_left(obj_t *obj){
    int i;
    
    if (CheckEMap(Ltx(obj)+1, Lty(obj), EM_BLOCK_LEFT)) return 1;
    //    printf("CHK_BLK_LEFT (%d, %d) height:%d\n", Ltx(obj), Lty(obj), obj->height);

    //    if (obj->height > 3){
    if (LbxOfst(obj)){
        // ʤ褦ˤ뤿height-2
        for (i = 1 ; i < obj->height-2 ; i++){
            if (CheckEMap(Ltx(obj), Lty(obj)+i, EM_BLOCK_LEFT)) return 1;
        }
        //        if (CheckEMap(Lbx(obj)+1, Lby(obj)-1, EM_BLOCK_LEFT)) return -1;
        if (CheckEMap(Lbx(obj)+1, Lby(obj), EM_BLOCK_LEFT)) return -1;
    }
    else {
        for (i = 0 ; i < obj->height-2 ; i++){
            if (CheckEMap(Ltx(obj), Lty(obj)+i, EM_BLOCK_LEFT)) return 1;
        }
        if (CheckEMap(Lbx(obj), Lby(obj), EM_BLOCK_LEFT)) return -1;
    }
    return 0;
    //    printf("cblock_left: %d %d %d  %d\n", Lbx(obj), Lby(obj), LbxOfst(obj), CheckEMap(Lbx(obj)+LbxOfst(obj), Lby(obj), EM_BLOCK_LEFT));
            //    for (i = 0 ; i < obj->height-2 ; i++){
}
int check_creature_block_right(obj_t *obj){
    int i;

    if (CheckEMap(Rtx(obj)-1, Rty(obj), EM_BLOCK_RIGHT)) return 1;
    //    for (i = 0 ; i < obj->height-2 ; i++){
    //    if (obj->height > 3){
    if (LbxOfst(obj)){
        for (i = 1 ; i < obj->height-2 ; i++){
            if (CheckEMap(Rtx(obj), Rty(obj)+i, EM_BLOCK_RIGHT)) return 1;
        }
        if (CheckEMap(Rbx(obj)-1, Rby(obj), EM_BLOCK_RIGHT)) return -1;
    }
    else {
        for (i = 0 ; i < obj->height-2 ; i++){
            if (CheckEMap(Rtx(obj), Rty(obj)+i, EM_BLOCK_RIGHT)) return 1;
        }
        if (CheckEMap(Rbx(obj), Rby(obj), EM_BLOCK_RIGHT)) return -1;
    }
    return 0;
    //    printf("cblock_right: %d %d %d  %d\n", Rbx(obj), Rby(obj), RbxOfst(obj), CheckEMap(Rbx(obj)-RbxOfst(obj), Rby(obj), EM_BLOCK_LEFT));
    //    if (CheckEMap(Rbx(obj)-RbxOfst(obj), Rby(obj)-1, EM_BLOCK_RIGHT)) return 1;
}

int check_creature_exclude_left(obj_t *obj){
    int i;

    // ϽŤʤƤ
    if (CheckEMap(Rtx(obj)-1, Rty(obj), EM_EXCLUDE_LEFT)) return 1;
    /*
    if (IsLanded(obj)){
        if (CheckEMap(Rtx(obj)-1, Rty(obj), EM_EXCLUDE_LEFT)) return 1;
    }
    else {
        if (CheckEMap(Rtx(obj), Rty(obj), EM_EXCLUDE_LEFT)) return 1;
    }
    */
    if (LbxOfst(obj)){
        for (i = 1 ; i < obj->height-2 ; i++){
            if (CheckEMap(Rtx(obj), Rty(obj)+i, EM_EXCLUDE_LEFT)) return 1;
        }
        // Ϸb . #פǰषʤˡ2ñʬ­եåȤĤȽ
        if (CheckEMap(Rbx(obj)-1, Rby(obj)-1, EM_EXCLUDE_LEFT)) return -1;
        if (CheckEMap(Rbx(obj)-1, Rby(obj), EM_EXCLUDE_LEFT)) return -1;
    }
    else {
        for (i = 1 ; i < obj->height ; i++){
            if (CheckEMap(Rtx(obj), Rty(obj)+i, EM_EXCLUDE_LEFT)) return 1;
        }
    }
    return 0;
    //	for (i = 0 ; i < obj->height-LbxOfst(obj) ; i++){
}
int check_creature_exclude_right(obj_t *obj){
    int i;

    // ϽŤʤƤ
    if (CheckEMap(Ltx(obj)+1, Rty(obj), EM_EXCLUDE_RIGHT)) return 1;
    /*
    if (IsLanded(obj)){
        if (CheckEMap(Ltx(obj)+1, Rty(obj), EM_EXCLUDE_RIGHT)) return 1;
    }
    else {
        if (CheckEMap(Ltx(obj), Rty(obj), EM_EXCLUDE_RIGHT)) return 1;
    }
    */
    if (LbxOfst(obj)){
        for (i = 1 ; i < obj->height-2 ; i++){
            if (CheckEMap(Ltx(obj), Lty(obj)+i, EM_EXCLUDE_RIGHT)) return 1;
        }
        if (CheckEMap(Lbx(obj)+1, Lby(obj)-1, EM_EXCLUDE_RIGHT)) return -1;
        if (CheckEMap(Lbx(obj)+1, Lby(obj), EM_EXCLUDE_RIGHT)) return -1;
    }
    else {
        for (i = 1 ; i < obj->height ; i++){
            if (CheckEMap(Ltx(obj), Lty(obj)+i, EM_EXCLUDE_RIGHT)) return 1;
        }
    }
    return 0;
        //	for (i = 0 ; i < obj->height-LbxOfst(obj) ; i++){
}
int check_creature_exclude_up(obj_t *obj){
    int x;

    for (x = LandLx(obj) ; x <= LandRx(obj) ; x++)
    {
        if (CheckEMap(x, Lby(obj), EM_EXCLUDE_UP)) return 1;
    }
//    puts("landed no");
    return 0;
}
int check_creature_exclude_down(obj_t *obj){
    int x, lx, rx;

    // ǲʤ顢
    // ˶ϩȤä󲼤褦ˡˤƤ
    if (!IsLanded(obj) && obj->u.creature.float_time == 0){
        lx = Ltx(obj);
        rx = Rtx(obj);
    }
    else {
        lx = Ltx(obj)+1;
        rx = Rtx(obj)-1;
    }

    for (x = lx ; x <= rx ; x++)
    {
        if (CheckEMap(x, Lty(obj), EM_EXCLUDE_DOWN)) return 1;
    }
    return 0;
    //    return check_emap_obj_any_up(obj, EM_EXCLUDE_DOWN);
}

exclude_result_t exclude_obj_horizontal(obj_t *obj){
	int left_moved = 0, right_moved = 0;
    //    while (check_emap_obj_any_right(obj, EM_EXCLUDE_LEFT)){
    while (check_creature_exclude_left(obj)){
        //        printf("exclude_left\n");
        obj->x --;
		left_moved ++;
    }
    //    while (check_emap_obj_any_left(obj, EM_EXCLUDE_RIGHT)){
    while (check_creature_exclude_right(obj)){
        //        printf("exclude_right\n");
        obj->x ++;
		right_moved ++;
    }
	if (CheckAttr(obj, at_rooted)){ // ꥿פʤ麸˲ʤ
		// ư餽ʬ٤줿Ȥߤʤ
		// 麸ɤ˲դ줿⤷ʤ
		// ξⱦ鲡줿ʬϱưϤʤΤǡ
		// ٤줿̤right_movedȽǤǤ롣
		// ٤쥰եåκüexclude_right줿֤Ǥ褤
		if (right_moved > 1){
            return er_smash_h2;
		}
		else if (right_moved == 1){
            return er_smash_h1;
		}
		// ưʤä顢鲡ƤʤΤ߲줿
		// Ԥξ硢֤򸵤ᤷ٤̤ꤹ
		else {
			obj->x += left_moved;
			if (left_moved > 1){
                return er_smash_h2;
			}
			else if (left_moved == 1){
                return er_smash_h1;
			}
		}
	}
    //    if (check_emap_obj_any_right(obj, EM_EXCLUDE_LEFT)){
    //        if (check_emap_box_any_right(obj->x-1, obj->y, obj->width, obj->height,
    //                                    EM_EXCLUDE_LEFT))
    if (check_creature_exclude_left(obj)){
        printf("ex left\n");
        // check_creature_exclude_leftȤ٤ɸ1ƤΤ
        // LandOfstȤäƤʤǤȽsmashh1/h2Τɤˤʤ뤫
        // 餢ޤʤϤ
        if (check_emap_box_any_right(obj->x-1, obj->y, obj->width, obj->height,
                                     EM_EXCLUDE_LEFT))
        {
            return er_smash_h2;
        }
        return er_smash_h1;
    }
    if (left_moved || right_moved){
        return er_excluded_horizontal;
    }
    return er_none;
}
exclude_result_t exclude_obj_vertical(obj_t *obj){
    //    while (check_emap_obj_any_down(obj, EM_EXCLUDE_UP)){
    while (check_creature_exclude_up(obj)){
        obj->y --;
        //        printf("exclude up %d\n", obj->y);
    }
//    while (check_emap_obj_any_up(obj, EM_EXCLUDE_DOWN)){
    while (check_creature_exclude_down(obj)){
        obj->y ++;
        //        printf("exclude down %d\n", obj->y);
    }
    //    if (check_emap_obj_any_down(obj, EM_EXCLUDE_UP)){
    if (check_creature_exclude_up(obj)){
        printf("smash v1 or v2\n");
        // check_creature_exclude_leftȤ٤ɸ1ƤΤ
        // LandOfstȤäƤʤǤȽsmashh1/h2Τɤˤʤ뤫
        // 餢ޤʤϤ
        if (check_emap_box_any_down(obj->x, obj->y-1, obj->width, obj->height,
                                    EM_EXCLUDE_UP))
        {
            //            printf("smash v2\n");
            return er_smash_v2;
        }
        //        printf("smash v1\n");
        return er_smash_v1;
    }
    return er_none;
}


// եȤʤɤˤǤϡ֤
// פʾer_none, ǤȤer_excluded_horizontal/vertial
// Ǥʤ(٤줿)Ȥer_smash_xx֤
exclude_result_t exclude_obj(obj_t *obj){
    exclude_result_t ret;

    // ˹⤵꤮η֤Ȥȡ
    // ­եåȤδطǺƤ뤿ᡢ
    // ư­줬ȤϾ岼˽ƲĤƤΤͥ褷Ƥ
	// MoveX==0Τϡ­Ƨ߳ƤҤääʤΤɤ
    if ((CheckEMap(Lbx(obj), Lby(obj), EM_EXCLUDE_RIGHT) && MoveX(obj) == -1) ||
        (CheckEMap(Rbx(obj), Rby(obj), EM_EXCLUDE_LEFT) && MoveX(obj) == 1))
            //    if(IsLanded(obj))
    //    if ((CheckEMap(LandLx(obj), Lby(obj), EM_EXCLUDE_RIGHT) && MoveX(obj) == -1) ||
    //        (CheckEMap(LandRx(obj), Rby(obj), EM_EXCLUDE_LEFT) && MoveX(obj) == 1))
    {
        ret = exclude_obj_vertical(obj);
        if (IsSmashed(ret)) return ret;
        //        if (ret != os_invalid) return ret;
        ret |= exclude_obj_horizontal(obj);
    }
    // 徺եȤüˤȤ⡢ͥƤʤ
    else if (CheckEMap(LandLx(obj), Lby(obj)-1, EM_PUSH_UP) ||
             CheckEMap(LandRx(obj), Rby(obj)-1, EM_PUSH_UP))
    {
        //        printf("on upward lift\n");
        ret = exclude_obj_vertical(obj);
        if (IsSmashed(ret)) return ret;
        ret |= exclude_obj_horizontal(obj);
    }
    else {
        ret = exclude_obj_horizontal(obj);
        //                printf("*1st ex-hori %d\n", ret);
//		if (ret == os_invalid){
		if (ret == er_none){
//		if (!IsSmashed(ret)){
	        ret |= exclude_obj_vertical(obj);
		}
		// ꥿פξ硢˲줿¨˴
		// ǻ˴ˤƤʤȺɸѤäƤ뤿ᡢ
		// 2ܤexclude_obj_horizontalǲ줿Ƚ꤬ʤ
       	else if (IsRooted(obj)) return ret;
		else {
            //            	        ret = exclude_obj_vertical(obj);
            ret |= exclude_obj_vertical(obj);
            //            		    printf("*1st ex-ver %d\n", ret);
	        if (IsSmashed(ret)) return ret;
		    ret |= exclude_obj_horizontal(obj);
            //            			printf("*2nd ex-hori %d\n", ret);
		}

    }
    //        printf("exclude obj ret:%d\n", ret);
    return ret;
}

// ԥסڶΥ֥Ȥ򺸱˰ư
// ˰ưǤ0Ǥʤ0֤
int move_obj_horizontal(obj_t *obj, int xoffset){
    if (xoffset < 0){
        // ¦˾㳲ʪʤаư
        //        if (!check_emap_dir(EM_BLOCK_LEFT, di_left, obj)){
        switch (check_creature_block_left(obj)){
        case 1:
            break;
        case -1:
            // LandLxʬǰưǽʤȤϡŷͥåƤʤȡ
            // ֤exclude_upƻ
            if (!check_emap_box_any_up(Ltx(obj)-1, Lty(obj), obj->width,
                                       obj->height, EM_BLOCK_UP))
            {
                obj->x --;
            }
            return 1;
        case 0:
            obj->x --;
            return 1;
        }
    }
    else if (xoffset > 0){
        // ¦˾㳲ʪʤаư
        //        if (!check_emap_dir(EM_BLOCK_RIGHT, di_right, obj)){
        switch (check_creature_block_right(obj)){
        case 1:
            break;
        case -1:
            // LandLxʬǰưǽʤȤϡŷͥåƤʤȡ
            // ֤exclude_upƻ
            if (!check_emap_box_any_up(Ltx(obj)+1, Lty(obj), obj->width,
                                       obj->height, EM_BLOCK_UP))
            {
                obj->x ++;
            }
            return 1;
        case 0:
            obj->x ++;
            return 1;
        }
    }
    return 0;
}

// ԥפΥ֥Ȥ򺸱⤫
// ϤƤ뤳Ȥ󡣳ʤϼưǾ롣
// ⤤­줬ʤȤäƤΤǡ
// ԾְʳǻȤäƤϤʤʤ
// ⤱0⤱ʤ0֤
int walk_obj(obj_t *obj, int xoffset){
//    printf("walk (%d, %d) xoffset=%d %d %d\n", obj->x, obj->y, xoffset,
//           CheckEMap(Lbx(obj), Lby(obj), eb_block_left),
//           check_emap_dir(eb_block_up, di_up, obj));
	SetWalkBlocked(obj, 0);
    if (xoffset == 0){
		return 0;
	}
    else if (xoffset < 0){
        switch (check_creature_block_left(obj)){
		// κ¦˾㳲ʪʤ
        case 0:
            obj->x --;
            // ⤤­줬ʤСΥ1ñ
            if (!check_creature_landed(obj)){
                obj->y ++;
            }
            return 1;
        // 
        case 1:
            break;
        // ɤϤʤ˳          
        case -1:
	        if (CanClimb(obj) &&
                // check_creature_block_leftǤϼ̤ⳬʤ˸Τǡ
                // ǳФ򥭥󥻥뤹
                !CheckEMap(LandLx(obj)-1, Lby(obj)-1, EM_PUSH_RIGHTDOWN) &&
                // Ƭξ㳲ʪå
                // @note 1ñ̤νŤʤϵƤΤLtx(obj)-1ȤϤʤ
                !check_emap_box_any_up(Ltx(obj), Lty(obj), obj->width-1,
                                       obj->height, EM_BLOCK_UP))
  	        {
                //                if (check_emap_box_any_up(Ltx(obj)-1, Lty(obj), obj->width,
                if (check_emap_box_any_up(Ltx(obj)-1, Lty(obj), obj->width,
                                          obj->height, EM_BLOCK_LEFT))
                {
                    SetWalkBlocked(obj, 1);
                }
                //                printf("walked up(%d, %d)\n", obj->x, obj->y);
				obj->x --;
				obj->y --;
				return 1;
            }
            break;
        default:
            assert(0);
        }
    }
	else {
        switch (check_creature_block_right(obj)){
		// α¦˾㳲ʪʤ
        case 0:
            obj->x ++;
            // ⤤­줬ʤСΥ1ñ
            if (!check_creature_landed(obj)){
                obj->y ++;
            }
            return 1;
        // 
        case 1:
            break;
        // ɤϤʤ˳          
        case -1:
	        if (CanClimb(obj) &&
                // check_creature_block_rightǤϼ̤ⳬʤ˸Τǡ
                // ǳФ򥭥󥻥뤹
                !CheckEMap(LandRx(obj)+1, Lby(obj)-1, EM_PUSH_LEFTDOWN) &&
                !check_emap_box_any_up(Ltx(obj), Lty(obj), obj->width-1,
                                       obj->height, EM_BLOCK_UP))
  	        {
                if (check_emap_box_any_up(Ltx(obj)+1, Lty(obj), obj->width,
                                          obj->height, EM_BLOCK_RIGHT))
                {
                    SetWalkBlocked(obj, 1);
                }
                //                printf("walked up(%d, %d)\n", obj->x, obj->y);
				obj->x ++;
				obj->y --;
				return 1;
            }
            break;
        default:
            assert(0);
        }
	}
	// ޤǤreturnƤʤо㳲ʪˤ֤Ĥä
	SetWalkBlocked(obj, 1);
	return 0;
	
}

// ζư
// !!! ȤꤢեѤʤΤǡưȤ­(down)åʤ
//     ʤɤʤ顢4å褦˽
// ư̤֤
int push_horizontal(obj_t *obj){
	int x, push_x=0;
    if (IsLanded(obj)){
//		for (x = Lbx(obj) ; x <= Rbx(obj) ; x++){
		for (x = LandLx(obj) ; x <= LandRx(obj) ; x++){
			// ­줬
			if (CheckEMap(x, Lby(obj), EM_BLOCK_DOWN)){
				// եȾʤȤꤢư
				// push_left/rightŤʤꤷ
				// ٤ƲûƤ
				// 껦ʤ鶯ưʤǤ褤
				if (CheckEMap(x, Lby(obj), EM_PUSH_LEFT)){
					push_x --;
				}
				else if (CheckEMap(x, Lby(obj), EM_PUSH_RIGHT)){
					push_x ++;
				}
				// եȤǤʤ­ˤäƤʤ
				// (⤷ꤵƤʤ)ư
				// åϽλ
				else {
					push_x = 0;
					break;
				}
			}
		}
	}
	if (push_x == 0){
        SetInertiaX(obj, 0);
	    return 0;
	}
	else if (push_x < 0){
		if (!check_emap_dir(EM_BLOCK_LEFT, di_left, obj)){
            //	        printf ("push left\n");
		    obj->x --;
            //			// ξΤ(줫饸פݤ)
            //			if (IsLanded(obj)){
		        SetInertiaX(obj, -1);
                //                printf("inertia_x=%d\n", InertiaX(obj));
                //            }
			return -1;
		}
	}
	else {
		if (!check_emap_dir(EM_BLOCK_RIGHT, di_right, obj)){
            //	        printf ("push right\n");
		    obj->x ++;
            //			// ξΤ(줫饸פݤ)
            //			if (IsLanded(obj)){
		        SetInertiaX(obj, 1);
                //                printf("inertia_x=%d\n", InertiaX(obj));
                //            }
	        return 1;
		}
	}
	return 0;
}


// ̤ζư
// ư̤֤
int push_diagonal(obj_t *obj){
  if (CheckEMap(LandRx(obj), Rby(obj), EM_PUSH_LEFTDOWN)){
//    if (CheckEMap(Rbx(obj), Rby(obj), EM_PUSH_LEFTDOWN)){
        // ʹ˿ʤȤ
        if (!check_emap_dir(EM_BLOCK_LEFT, di_left, obj)){
            puts("slide left");
            obj->x--;
            obj->y++;
        }
        // פæФǤʤ褦̾ʤ֥åƳʤȤ
        // sliding
        SetSliding(obj, 1);
        //		SetLanded(obj, 1);
        return 1;
    }
  else if (CheckEMap(LandLx(obj), Lby(obj), EM_PUSH_RIGHTDOWN)){
      //    else if (CheckEMap(Lbx(obj), Lby(obj), EM_PUSH_RIGHTDOWN)){
        // ʹ˿ʤȤ
		if (!check_emap_dir(EM_BLOCK_RIGHT, di_right, obj)){
            puts("slide right");
            obj->x++;
            obj->y++;
        }
        // פæФǤʤ褦̾ʤ֥åƳʤȤ
        // sliding
		SetSliding(obj, 1);
        //		SetLanded(obj, 1);
        return 1;
    }
	SetSliding(obj, 0);
    return 0;
}

// äˤФ
// ФϽȡФζư
// Фʤ1֤
int blast_horizontal(obj_t *obj){
// ƱؤοФʤꤷʤ
// ƱEM_BLAST2ե졼ʾƧǽꡢblast_countʣΤɤᡣ
// ñblast_x!=0ǤϡФ˵ؤοФ줬ȯʤʤ
// ФƱؤοФ줬Ƥ̵뤵뤬ɤΤߤ̤ʤʤ
    int is_new_blast = 0;

	if (check_emap_obj_any_rim(obj, EM_BLAST_LEFT) &&
        (obj->u.creature.blast_x != -2))
    {
		puts("blast left");
		SetState(obj, os_blasted);
		SetBlastX(obj, -2);
        is_new_blast = 1;
        //		SetStunned(obj, 1);
        //		if (obj->type == ot_chee){
        //        IncGameRecord(re_blast_count);
            //		}
	}
	else if (check_emap_obj_any_rim(obj, EM_BLAST_RIGHT) &&
             (obj->u.creature.blast_x != 2))
    {
		puts("blast right");
		SetState(obj, os_blasted);
		SetBlastX(obj, 2);
        is_new_blast = 1;
        //		SetStunned(obj, 1);
        //		if (obj->type == ot_chee){
        //        IncGameRecord(re_blast_count);
            //		}
	}
	if (BlastX(obj) < 0){
		puts("blasting left");
		if (!check_emap_dir(EM_BLOCK_LEFT, di_left, obj)){
			obj->x --;
            // blasting endȤʤΤϼºݤˤϿǤʤΤǡ
            // ǽΰưǥȤ
            if (is_new_blast){
                IncGameRecord(re_blast_count);
            }
		}
		else {
			puts("blasting left end");
			SetState(obj, os_none);
			SetBlastX(obj, 0);
            BlastedShockFreezeLeft;
		}
		if (!check_emap_dir(EM_BLOCK_LEFT, di_left, obj)){
			obj->x --;
		}
		else {
			puts("blasting left end");
			SetState(obj, os_none);
			SetBlastX(obj, 0);
            BlastedShockFreezeLeft;
		}
		return 1;
	}
	else if (BlastX(obj) > 0){
		puts("blasting right");
		if (!check_emap_dir(EM_BLOCK_RIGHT, di_right, obj)){
			obj->x ++;
            if (is_new_blast){
                IncGameRecord(re_blast_count);
            }
		}
		else {
			puts("blasting right end");
			SetState(obj, os_none);
			SetBlastX(obj, 0);
            //			SetStunned(obj, 0);
            BlastedShockFreezeRight;
            //			set_emap_box(Rtx(obj)+1, Rty(obj), 4, obj->height, eb_shock);
            //            CheckBlastedRightToIce;
		}
		if (!check_emap_dir(EM_BLOCK_RIGHT, di_right, obj)){
			obj->x ++;
		}
		else {
			puts("blasting right end");
			SetState(obj, os_none);
			SetBlastX(obj, 0);
            //			SetStunned(obj, 0);
            //			set_emap_box(Rtx(obj)+1, Rty(obj), 4, obj->height, eb_shock);
            //            CheckBlastedRightToIce;
            BlastedShockFreezeRight;
		}
		return 1;
	}
	return 0;
}

// ƥݡȤ򳫻
int start_teleporting_creature(obj_t *obj){
    obj_t *target1, *target2;

    if (CheckEMap(Ltx(obj), Lty(obj), EM_TELEPORT) &&
        CheckEMap(Rtx(obj), Rty(obj), EM_TELEPORT))
    {
        target1 = GetEMapOwner(Ltx(obj), Lty(obj), _teleport);
        target2 = GetEMapOwner(Rtx(obj), Rty(obj), _teleport);
        assert(target1 != NULL);
        assert(target2 != NULL);
        if (target1 == target2){
            PushState(obj, os_teleporting);
			obj->link = target1;
//            obj->x = target1->link->x + obj->x - Ltx(target1);
//            obj->y = target1->link->y + obj->y - (Lby(target1)+1);
            return 1;
        }
    }
    return 0;
}

// ƥݡȳϸ1ե졼ʬž
int teleport_creature(obj_t *obj){
	obj_t *teleporter, *dest;

	IncCtr(obj);
    printf("TELEPORTING CTR=%d h=%d\n", Ctr(obj), obj->height);
    if (Ctr(obj) > obj->height<<1){
        PopState(obj);
        teleporter = obj->link;
        dest = teleporter->link;

        /*
        while (dest != NULL){
            if (dest->type == ot_teleport_dest){
                break;
            }
            else if (dest->type == ot_teleporter){
                if (dest->u.hub.in_num != 1){
                    log_error("Teleporter(%d, %d) is linked to "
                              "multiple-output hub (%d, %d).\n",
                              GridX(teleporter), GridY(teleporter),
                              GridX(dest), GridY(dest));
                    exit(1);
                }
                dest = dest->u.hub.out_objs[0];
            }
        }
        */
        if (dest == NULL){
            log_error("Teleporter(%d, %d) is not linked to dest.\n",
                      GridX(teleporter), GridY(teleporter));
            exit(1);
        }
        obj->x = teleporter->link->x + obj->x - Ltx(teleporter);
        obj->y = teleporter->link->y + obj->y - (Lby(teleporter)+1);
        assert(obj->x >= MAP_LTX); 
        assert(obj->y >= MAP_LTY);
        assert(obj->x <= MAP_RBX); 
        assert(obj->y <= MAP_RBY);
		IncGameRecord(re_teleport_count);
        return 1;
    }
    return 0;
}

void set_motion_creature(obj_t *obj){
	// ޤŪơ֥Υ⡼
	switch (State(obj)){
	case os_smash_h1:
		obj->motion = (obj->dir == di_left) ?
            creature_image_table[obj->type].smash_h1_l :
            creature_image_table[obj->type].smash_h1_r;  
		break;
	case os_smash_h2:
		obj->motion = (obj->dir == di_left) ?
            creature_image_table[obj->type].smash_h2_l :
            creature_image_table[obj->type].smash_h2_r;  
		break;
	case os_smash_v1:
		obj->motion = (obj->dir == di_left) ?
            creature_image_table[obj->type].smash_v1_l :
            creature_image_table[obj->type].smash_v1_r;  
		break;
	case os_smash_v2:
		obj->motion = (obj->dir == di_left) ?
            creature_image_table[obj->type].smash_v2_l :
            creature_image_table[obj->type].smash_v2_r;  
		break;
	case os_pierced:
		obj->motion = (obj->dir == di_left) ?
            creature_image_table[obj->type].pierced_l :
            creature_image_table[obj->type].pierced_r;  
		break;
	case os_burned:
		obj->motion = (obj->dir == di_left) ?
            creature_image_table[obj->type].burned_l :
            creature_image_table[obj->type].burned_r;  
		break;
	case os_electrocuted:
		obj->motion = (obj->dir == di_left) ?
            creature_image_table[obj->type].electrocuted_l :
            creature_image_table[obj->type].electrocuted_r;  
		break;
	case os_blasted:
		obj->motion = (obj->dir == di_left) ?
            creature_image_table[obj->type].blasted_l :
            creature_image_table[obj->type].blasted_r;  
		break;
	case os_digested:
		obj->motion = ii_none;
		break;
	case os_bang:
        switch (Size2Id(obj->width, obj->height)){
        case Size2Id(12, 8):
            obj->motion = ii_bang12_8;
            break; 
        case Size2Id(8, 16):
            obj->motion = ii_bang8_16;
            break; 
        case Size2Id(8, 8):
            obj->motion = ii_bang8_8;
            break; 
        case Size2Id(8, 4):
            obj->motion = ii_bang8_4;
            break; 
        default: // 4x4, 4x3
            obj->motion = ii_bang4_4;
            break; 
        }
        //		obj->motion = creature_image_table[obj->type].bang;
		break;
	default:
		break;
	}

	// ͭؿФǾ񤭤
	if (obj_funcs_table[obj->type].set_motion){
        obj_funcs_table[obj->type].set_motion(obj);
    }
}

int setup_creature(obj_t *obj){
	SetPrevLanded(obj, IsLanded(obj));
    if (check_creature_landed(obj)){
        SetLanded(obj, 1);
        SetWalking(obj, 1);
    }
    if (obj_funcs_table[obj->type].set_emap){
        obj_funcs_table[obj->type].set_emap(obj);
    }
    // setup̵weight
    // @note ONaswitchʪϽ֤⤯ȤˤʤΤǡ
    // Ȥꤢweight°Ĥʤsetup_aswitchǤΰȯưʤ
    // ϻ⤤Ƥ硢weight°ϱƶΤʤ֤ˤȯʤ
    // ³ϤʤϤ
	set_emap_owner_box(LandLx(obj), Lby(obj), LandWidth(obj), 1,
                       obj, en_weight);
    return 0;
}

int act_creature(obj_t *obj){
    exclude_result_t er;

	SetPrevLanded(obj, IsLanded(obj));

    switch (State(obj)){
    case os_teleporting:
        teleport_creature(obj);
        return 0;
    case os_smash_h1:
    case os_smash_h2:
    case os_smash_v1:
    case os_smash_v2:
        printf("smash:%d\n", State(obj));
        SetState(obj, os_bang);
        break;
    case os_eaten:
        SetState(obj, os_digested);
        break;
    case os_digested:
        IncCtr(obj);
		//  òˤݤǽ⤢뤬
		//    줿creature¦ǥȥåפƤΤǡ
		//    ɬ˴롣
		//    ߹ޤ줿ʹߤˤ𤬻ǤʤΤ
		//    εưʤ
		//    ̩ˤϤ𤬻¨¤creature٤
		//    os_digestedcreatureϲ⤷ʤ³ʤ
        if (Ctr(obj) == OJIGIRI_DIGEST_COUNT){
            puts("dead digested");
			SetState(obj, os_dead);
//			inc_kill_record(obj->type);
//            return 1;
        }
        break;
    case os_bang:
		SetState(obj, os_dead);
        return 0;
    case os_dead:
		inc_kill_record(obj->type);
        return 1;
	case os_pierced:
	case os_burned:
	case os_electrocuted:
		IncCtr(obj);
        //  		printf("pierced/burnd/electrocuted: ctr:%d\n", Ctr(obj)); 
        if (State(obj) == os_burned || State(obj) == os_electrocuted){
            if (Ctr(obj) >= CREATURE_DEATH_COUNT){
                // ǳŤȤbangʤ
                SetState(obj, os_dead);
                break;
			}
            else {
                if (IsTurnToMist(obj)) return 0;
            }
        }
        else {
	        if (Ctr(obj) >= CREATURE_DEATH_COUNT){
				SetState(obj, os_bang);
			    break;
			}
		}
        /* FALLTHRU */
    default:
        // ʤΤ˱˿줿˴
        // Хꥢexclude뤫顢exclude_obj˽Ťʤ˴ȽꤷƤ
		if (State(obj) != os_burned){
            if (!IsImmuneFlame(obj) && check_emap_obj_any(obj, EM_FLAME)){
                //	        if (check_emap_any_rim(eb_flame, obj)){
				// Chuʤɤ
				// ǳξˤburnedmotionѤ뤿ᡢ
                // PushStateǵ֤¸
                // бPopStateϼ¹Ԥʤburnϻ̤ʤ
		        PushState(obj, os_burned);        
			    break;
			}
            if (!IsImmuneWater(obj) && check_emap_obj_any(obj, EM_WATER)){
		        PushState(obj, os_burned);        
			    break;
			}
		}
        if (State(obj) != os_electrocuted){
			// ХꥢԹ塢ŷʪϵʤ
            if (check_emap_obj_any(obj, EM_ELECTRIC)){
		        PushState(obj, os_electrocuted);
			    break;
			}
        }
        if (start_teleporting_creature(obj)){
            return 0;
        }
        er = exclude_obj(obj);
        if (IsSmashed(er)){
            SetState(obj, ExcludeResult2SmashState(er));
            // ٤줿ϰʲλ˴ưϥåפ⡼
            break;
        }
        /*
        // Ф줿٤Ƥʤϡɤˤ֤Ĥäˤ
        else if (IsExcludedHorizontal(er)){
            printf("walk blocked by excluding\n");
            //            SetWalkBlocked(obj, 1);
        }
        */
        /*
        if (sm){
            SetState(obj, sm);
            // ٤줿ϰʲλ˴ưϥåפ⡼
            break;
        }
        */
		// ˿줿˴
        if (State(obj) != os_pierced){
			if (!CheckAttr(obj, at_immune_pierce) &&
				check_emap_any_rim(EM_PIERCE, obj))
			{
				SetState(obj, os_pierced);
	        }
		}
        // եȤǺư
        push_horizontal(obj);

        // ǤϾis_landed򹹿
        SetLanded(obj, check_creature_landed(obj));

        // set_gondin_eaten줿Ǥ󤬾äʤɡ
        // set_action!IsAliveΤȤνޤिᡢ
        // AliveǤʤƤƤӽФ
        // Τλưʤ褦ˡset_action¦դɬ
        // walk_objθƤӽФ!IsAliveå򤷤Ƥ뤫顢
        // Τ⤯ȤϤʤ
        if (obj_funcs_table[obj->type].set_action){
            if (obj_funcs_table[obj->type].set_action(obj)) return 1;
        }
		// ä˿Ф
		if (blast_horizontal(obj)){
		    // Ǥϡblast_horizontalǽƤޤΤǡ϶
		}
        // ̤
        else if (push_diagonal(obj)){
            // Ǥϡpush_diagonalǽƤޤΤǡ϶
        }
        // к⤯
        else if (IsLanded(obj)){
        //        else if (check_creature_landed(obj)){
        //        else if (check_emap_dir(eb_block_down, di_down, obj)){
            //            printf("creature checked landed state=%d\n", State(obj));
            //            SetLanded(obj, 1);
			// set_actionǾ֤Ѳ뤳ȤΤǡưǽåɬ
            if (IsWalkable(obj)){
                //                puts("checked walkable");
                //                obj->u.creature.is_walking =
                SetWalking(obj, walk_obj(obj, MoveX(obj)));
            }
        }
        // ʤпľ
        else {
            //            puts("creature falling");
            obj->y ++;
            if (check_creature_landed(obj)){
                //            puts("creature landed");
                SetLanded(obj, 1);
				// ONaswitchΤܤæѤˡˤwalkɬ
				// PrevLandedȤäơ̾ˤϥ⤭Фʤ褦ˤ
                if (IsWalkable(obj) && IsPrevLanded(obj)){
                    walk_obj(obj, obj->u.creature.move_x);
                }   
            }
            else {
                SetLanded(obj, 0);
                move_obj_horizontal(obj, obj->u.creature.move_x);
            }
        }
    }
    // ưη̤кǡǸˤ⤦ٿʿexclude
    //    exclude_obj_horizontal(obj);
	set_motion_creature(obj);
    if (obj_funcs_table[obj->type].set_emap){
        obj_funcs_table[obj->type].set_emap(obj);
    }
    return 0;
}


void set_emap_creature(obj_t *obj){
    if (!IsAlive(obj)) return;
	if (CheckAttr(obj, at_meat)){
	    set_emap_owner_box(obj->x, obj->y, obj->width, obj->height,
		                   obj, en_meat);
	}
	else if (CheckAttr(obj, at_vegetable)){
        set_emap_box(obj->x, obj->y, obj->width, obj->height, EM_VEGETABLE);
// !! vegetableФReqStateʤ
//		    set_emap_owner_box(obj->x, obj->y, obj->width, obj->height,
//			                   eb_vegetable, obj);
	}
//	else if (CheckAttr(obj, at_item)){
//        set_emap_box(obj->x, obj->y, obj->width, obj->height, EM_ITEM);
// !! itemФReqStateʤ
//		    set_emap_owner_box(obj->x, obj->y, obj->width, obj->height,
//			                   eb_item, obj);
//	}
	if (IsLanded(obj) && !IsBlasted(obj)){
		set_emap_owner_box(LandLx(obj), Lby(obj), LandWidth(obj), 1,
                           obj, en_weight);
        //        printf("set weight[%d-%d]\n", LandLx(obj), LandLx(obj)+LandWidth(obj)-1);
	}
}



int act_chee(obj_t *obj){
    //    int move_x;
    //    obj_state_t st;
    exclude_result_t er;
	int v;
	obj_t *target;
//	printf("is_landed:%d\n", IsLanded(obj));
//    printf("chee state:%x\n", State(obj));

    // ˴ν
    //    if (!IsAlive(obj)){
        //        printf("dead state %d dir:%d ctr:%d\n", State(obj), obj->dir, Ctr(obj));
        switch (State(obj)){
        case os_teleporting:
            teleport_creature(obj);
            return 0;
        case os_smash_h1:
        case os_smash_h2:
        case os_smash_v1:
        case os_smash_v2:
			CountRecord(re_killed_by_smash);
            SetState(obj, os_bang);
            set_motion_creature(obj);
            return 0;
        case os_eaten:
			if (Ctr(obj) == 0){
				CountRecord(re_killed_by_ojigiri);
			}
            IncCtr(obj);
            if (Ctr(obj) == OJIGIRI_SWALLOW_COUNT){
                SetState(obj, os_digested);
                SetMotion(obj, ii_none); // ٤Ƥ֤ɽ
            }
            return 0;
        case os_digested:
            IncCtr(obj);
            if (Ctr(obj) == OJIGIRI_DIGEST_COUNT + OJIGIRI_VOMIT_COUNT){
                SetState(obj, os_ghost);
                set_motion_creature(obj);
            }
            return 0;
        case os_exit:
			IncCtr(obj);
//            obj->motion = ii_chee_back;
            if (Ctr(obj) == CHEE_EXIT_COUNT){
                SetState(obj, os_clear);
                SetMotion(obj, ii_none);
            }
            return 0;
		case os_clear:
			if (Ctr(obj) == 0) finalize_objective_stat();
			IncCtr(obj);
            if (Ctr(obj) == CHEE_CLEAR_COUNT){
                SetState(obj, os_show_result);
            }
            return 0;
        case os_pierced:
			if (Ctr(obj) == 0){
				CountRecord(re_killed_by_pierced);
			}
            IncCtr(obj);
            if (Ctr(obj) == CHEE_PIERCE_COUNT){
                SetState(obj, os_ghost);
            }
            set_motion_creature(obj);
            return 0;
        case os_burned:
			if (Ctr(obj) == 0){
				CountRecord(re_killed_by_flame);
			}
            IncCtr(obj);
            if (Ctr(obj) == CHEE_BURN_COUNT){
                SetState(obj, os_ghost);
            }
            set_motion_creature(obj);
            return 0;
        case os_electrocuted:
			if (Ctr(obj) == 0){
				CountRecord(re_killed_by_barrier);
			}
            IncCtr(obj);
            if (Ctr(obj) == CHEE_BURN_COUNT){
                SetState(obj, os_ghost);
            }
            set_motion_creature(obj);
            return 0;
        case os_bang:
            SetState(obj, os_ghost);
            set_motion_creature(obj);
            return 0;
        case os_ghost:
            obj->y --;
            IncCtr(obj);
            if (Rby(obj) == 0 || Ctr(obj) == GHOST_COUNT){
                SetState(obj, os_gameover);
            }
            return 0;
		case os_show_result:
		case os_gameover:
            IncCtr(obj);
            SetMotion(obj, ii_none); // ɽ
            return 0;
        default:
            break;
        }

    // ˿줿˴
    // !!! LandLx/RxкǥХꥢflame°ϰϤ򶹤Τǡ
    // any_rimȽǼꤳܤ
    // 㴳ŤobjϰΤȽꤷƤ
    //    if (check_emap_any_rim(eb_flame, obj)){
    if (check_emap_obj_any(obj, EM_FLAME)){
        SetState(obj, os_burned);        
        printf("burned!!!!\n");
        set_motion_creature(obj);
        return 0;
    }
    if (check_emap_obj_any(obj, EM_ELECTRIC)){
        SetState(obj, os_electrocuted);        
        printf("electrocuted!!!!\n");
        set_motion_creature(obj);
        return 0;
    }
    
    if (start_teleporting_creature(obj)){
        set_motion_creature(obj);
        return 0;
	}
    er = exclude_obj(obj);
    if (IsSmashed(er)){
        SetState(obj, ExcludeResult2SmashState(er));
        set_motion_creature(obj);
        return 0;
    }

    if (State(obj) == os_chee_eating){
        obj->motion = ii_chee_eat0;
        IncCtr(obj);
        //        printf("chee eating:%d\n", Ctr(obj));
        if (Ctr(obj) == CHEE_EAT_COUNT){
            SetState(obj, os_none);
            //            obj->u.creature.is_stunned = 0;
            // !! set_creature_motionΤ̵̣
            SetMotion(obj, ii_chee_init);
			if (inc_gondin_eaten()){
				ReqState(goal, obj, os_goal_req_open);
			}
        }
    }
    if ((v = check_emap_pos_obj_any_corner(obj, EM_SIGN)))
    {
        target = GetEMapOwner(CheckVal2PosX(v), CheckVal2PosY(v), _sign);
        assert(target != NULL);
        //		printf("found sign link=%d\n", target->u.sign.no);
		set_current_stage_sign_no(target->u.sign.no);
	}
	else {
		set_current_stage_sign_no(INVALID_LINK_NO);
	}

	if (IsWalkable(obj) &&
        ((v = check_emap_obj_any_rim(obj, EM_GONDIN)) ||
         (v = check_emap_box_any(obj->x+3, obj->y+3, 1, 1, EM_GONDIN))))
    {
        //        printf("v=%d\n", v);
        target = GetEMapOwner(CheckVal2PosX(v), CheckVal2PosY(v), _gondin);
        assert(target != NULL);
        ReqState(target, obj, os_gondin_eaten);
        puts("Gondin!!!");
        SetState(obj, os_chee_eating);
        // ٤Ƥ֤ϽϤŨι뤬ưǽʤΤ
        // stunnedˤƤ
        //        obj->u.creature.is_stunned = 1;
        obj->motion = ii_chee_eat0;
        return 0;
    }

    if (check_emap_obj_all_corner(obj, EM_GOAL) && IsWalkable(obj)){
        puts("Goal!!!");
        SetState(obj, os_exit);
        ReqState(goal, obj, os_exit);
        obj->motion = ii_chee_back;
        return 0;
    }
    // ˿줿˴
    if (check_emap_any_rim(EM_PIERCE, obj)){
        SetState(obj, os_pierced);        
        set_motion_creature(obj);
        return 0;
    }
	if (MoveX(obj) && IsWalkable(obj)){
		obj->dir = (MoveX(obj) < 0) ? di_left : di_right;
	}

    // եȤǺư
    if (push_horizontal(obj)){
		IncGameRecord(re_lift_frame);
	}

	// ä˿Ф
	if (blast_horizontal(obj)){
	}
    // 
    else if (obj->u.creature.float_time){
        //        printf("jumping jx=%d jy=%d\n", obj->u.creature.jump_x, obj->u.creature.jump_y);
        // 徺
//        if (obj->u.creature.jump_time){        
		if (obj->y > obj->u.creature.jump_y_limit){        
            if (!check_emap_dir(EM_BLOCK_UP, di_up, obj)){
                obj->y --;
                // եȤδʬû
                // ΤȤϡͤȴʤ褦1ñ̤ľȽ&ư
                if (obj->u.creature.jump_y < 0){
                    if (!check_emap_dir(EM_BLOCK_UP, di_up, obj)){
                        obj->y += obj->u.creature.jump_y;
                    }
                }
                else {
                    obj->y += obj->u.creature.jump_y;
                }
            }
            else {
                // ŷƬ֤ĤƤϤʤ
                // ˤ褱Ƥ⤽ʾϾ徺Ǥʤ
                //            obj->u.creature.jump_time = 0;
                // ˤäΥɤˤȡ
                //徺ŷ򤱤кƾ徺Ǥ褦ˤʤ
            }
            obj->u.creature.float_time --;
        }
        // 徺λڶ
        else {
            obj->u.creature.jump_y = 0;
            obj->u.creature.float_time --;
        }
        move_obj_horizontal(obj, obj->u.creature.move_x);
        
        // դʤ餽ʬ˰ư
        // ưǤʤä(=֤Ĥä)鴷ʬ®٤0ˤ
        if (obj->u.creature.jump_x){
            if (move_obj_horizontal(obj, obj->u.creature.jump_x) == 0){
                obj->u.creature.jump_x = 0;
            }
        }
        IncGameRecord(re_air_frame);
    }
    // ̤
    // פڶ˼̤ζư̤Фʤ褦ˡ
    // ׽θǽ
    else if (push_diagonal(obj)){
        printf("chee sliding %d\n", MoveX(obj));
        // Ǥϡpush_diagonalǽƤޤΤǡ϶
    }
    // Ǥʤư֥åƤʤϤȤߤʤ
    // ľǤ⡢жϤξǥ׻Ȥ
    //    else if (check_emap_dir(eb_block_down, di_down, obj)){
    else if (check_creature_landed(obj)){
        //        printf("landed walk:%d jump_x:%d\n", obj->u.creature.move_x, obj->u.creature.jump_x);
        SetLanded(obj, 1);
        SetJumpX(obj, 0);
//        obj->u.creature.jump_y = 0;
        if (IsWalkable(obj)){
            //                puts("chee checked walkable");            
            if (walk_obj(obj, MoveX(obj))){
				IncGameRecord(re_walk_frame);
			}
        }
    }
    else { // ڶʳǲư֥åƤʤȤ
//        		printf("falling %d %d\n", obj->u.creature.jump_time, obj->u.creature.float_time);
        obj->y ++;
        // 1ñ̡­줬landed
        if (check_creature_landed(obj)){
        //        if (check_emap_dir(eb_block_down, di_down, obj)){
            puts("LANDED");
            SetLanded(obj, 1);
            // ̤Ϥ硢sliding°ĤƤʤ
            // ˥פդƤޤ
            if (CheckEMap(LandRx(obj), Rby(obj), EM_PUSH_LEFTDOWN) || 
                CheckEMap(LandLx(obj), Lby(obj), EM_PUSH_RIGHTDOWN))
            {
                puts("LANDED ON SLOPE");
                SetSliding(obj, 1);
            }

            //			// ߥեȤꤷϤľϥפ˲
//			obj->u.creature.jump_y = 1;
	        if (IsWalkable(obj)){
		        walk_obj(obj, obj->u.creature.move_x);
			}
        }
        else { // ­줬ʤΤ
            SetLanded(obj, 0);
			move_obj_horizontal(obj, obj->u.creature.move_x);
            // դʤ餽ʬ˰ư
			   // ưǤʤä(=֤Ĥä)鴷ʬ®٤0ˤ
            if (obj->u.creature.jump_x){
                if (move_obj_horizontal(obj, obj->u.creature.jump_x) == 0){
                    obj->u.creature.jump_x = 0;
                }
            }
            IncGameRecord(re_air_frame);
        }
    }
    // ľեȾΤȤδ
    //  ʬʤɤαƶȿǤ뤿ᡢΰ֤Ƚꤹ
    if (check_emap_obj_any_down(chee, EM_PUSH_DOWN)){
        obj->u.creature.inertia_y = 1;
    }
    else if (check_emap_obj_any_down(chee, EM_PUSH_UP)){
        obj->u.creature.inertia_y = -1;
    }
    else {
        obj->u.creature.inertia_y = 0;
    }
    //    printf("inertia y:%d\n", obj->u.creature.inertia_y);

    // ǽŪˤ⤦excludeƤꤳߤɻ
    // !!! Ȥꤢ \C| ηǤΤꤳߤ˻ߡ
    // \C/٤Τ \C|٤褦ˤ
    // !!! Υɤ =>= C d  ηǳʤξ򤹤ȴƱ˲Ф
    //        exclude_obj_horizontal(obj);
    //            exclude_obj_vertical(obj);

    set_motion_creature(obj);
    //    printf("motion %d %d %d\n", obj->motion, ii_chee_jump_r, ii_chee_walk_r0);
    set_emap_chee(obj);
    return 0;
}

void set_motion_chee(obj_t *obj){
    switch (State(obj)){
	SkipDefaultCreatureMotions;
    case os_ghost:
        if (obj->dir == di_left){
            SetMotion(obj, ii_chee_ghost_l0);
        }
        else {
            SetMotion(obj, ii_chee_ghost_r0);
        }
        break;
    default:
        //        printf("default motion:%d %d\n", IsSliding(obj), MoveX(obj));
        // ϥϤʤС饤ǥ󥰻Ǹ
        // ϤСФ뤿̾ԥ˥᡼
        if (IsSliding(obj)){
            if (MoveX(obj) == 0){
                SetMotionDir(obj, ii_chee_jump_l, ii_chee_jump_r);
            }
            else {
                SetWalkMotion(obj);
            }
        }
        /*
    else if (obj->u.creature.blast_x){
        obj->motion = (obj->dir ==  di_left) ?
            ii_chee_blast_l : ii_chee_blast_r;
    }
        */
        else if (obj->u.creature.is_landed){
            if (MoveX(obj) == 0){
                SetMotionDir(obj, ii_chee_walk_l0, ii_chee_walk_r0);
            }
            else {
                SetWalkMotion(obj);
            }
            /*
            if (obj->u.creature.move_x < 0){
                obj->motion = (obj->motion == ii_chee_walk_l0) ?
                    ii_chee_walk_l1 : ii_chee_walk_l0;
            }
            else if (obj->u.creature.move_x > 0){
                obj->motion = (obj->motion == ii_chee_walk_r0) ?
                    ii_chee_walk_r1 : ii_chee_walk_r0;
            }
            else { // move_x == 0
                obj->motion = (obj->dir == di_left) ?
                    ii_chee_walk_l0 : ii_chee_walk_r0;
            }
            */
        }
        else { // not landed
            if (obj->dir == di_right){
                if (!jump_key_released && GetMotion(obj) == ii_chee_jump_r){
                    //            if (obj->motion == ii_chee_jr){
                    SetMotion(obj, ii_chee_flutter_r);
                }
                else {
                    SetMotion(obj, ii_chee_jump_r);
                }
            }
            else if (obj->dir == di_left){
                if (!jump_key_released && GetMotion(obj) == ii_chee_jump_l){
                    //            if (obj->motion == ii_chee_jl){
                    SetMotion(obj, ii_chee_flutter_l);
                }
                else {
                    SetMotion(obj, ii_chee_jump_l);
                }
            }
        }
    }
}

void set_emap_chee(obj_t *obj){
    set_emap_creature(obj);
    set_emap_box(obj->x, obj->y, obj->width, obj->height, EM_CHEE);
    /*
    set_emap_owner_box(obj->x, obj->y, obj->width, obj->height, eb_meat, obj);
	if (IsLanded(obj) && !IsBlasted(obj)){
		set_emap_box(LandLx(obj), Lby(obj), LandWidth(obj), 1, eb_weight);
		set_emap_owner_box(LandLx(obj), Lby(obj), LandWidth(obj),
                           1, eb_weight, obj);
	}*/
}


int set_action_gondin(obj_t *obj){
    if (State(obj) == os_gondin_eaten) return 1;
	return 0;
}
void set_motion_gondin(obj_t *obj){
    switch (State(obj)){
	SkipDefaultCreatureMotions;
    default:
        SetMotion(obj, ii_gondin);
        break;
    }
}
void set_emap_gondin(obj_t *obj){

    set_emap_creature(obj);
	// äȿ褦vegetable°ĤƤ
	// ojigiri_immuneʤΤǤ˿٤뤳ȤϤʤ
	if (IsAlive(obj)){ // burnedʤɤ˿٤ʤ褦en_gondinϷߤλΤ
        //        set_emap_box(obj->x+3, obj->y+3, 2, 2, en_gondin);
		set_emap_owner_box(obj->x+3, obj->y+3, 2, 2, obj, en_gondin);
	}
}


int set_action_hatopo(obj_t *obj){
	if (!IsAlive(obj)) return 0;
	if (IsStunned(obj)) return 0;
    if (IsLanded(obj)){
        // ⤱餽Τޤ⤭³
        if (!IsWalkBlocked(obj)){
            SetMoveXDir(obj);
        }
        // äƤϤʤˤ⤷ʤ
        else if (IsSliding(obj)){
        }
        // Ϥ⤱ʤϤ֤ĤäƤΤȿž
        else {
            obj->dir = ReverseDir(obj->dir);
            SetMoveXDir(obj);
        }
    }
    else { // 
        // Ԥ֥å줿ȤϸȿžƤ
        // 
        //        if (IsWalkBlocked(obj) && IsPrevLanded(obj)){
        if (IsWalkBlocked(obj)){
            obj->dir = ReverseDir(obj->dir);
        }
        SetMoveX(obj, 0);
    }
	return 0;
}
void set_motion_hatopo(obj_t *obj){
    switch (State(obj)){
	SkipDefaultCreatureMotions;
    default:
        if (obj->u.creature.is_sliding){
            SetMotionDir(obj, ii_hatopo_slide_l, ii_hatopo_slide_r);
        }
        // ­򳫤ޤ޸
        else if (!obj->u.creature.is_landed){
            SetMotionDir(obj, ii_hatopo_walk_l0, ii_hatopo_walk_r0);
        }
        // Ի
        else {
            SetWalkMotion(obj);
            /*
            if (obj->dir == di_left){
                obj->motion = (obj->motion == ii_hatopo_walk_l0) ?
                    ii_hatopo_walk_l1 : ii_hatopo_walk_l0;
            }
            else if (obj->dir == di_right){
                obj->motion = (obj->motion == ii_hatopo_walk_r0) ?
                    ii_hatopo_walk_r1 : ii_hatopo_walk_r0;
            }
            */
        }
        break;
    }
}
int set_action_yaramasi(obj_t *obj){
	if (!IsAlive(obj)) return 0;
	if (IsStunned(obj)) return 0;
	// ɤᡢat_immune_pierceˤƤơ
	// ʢ¦ΤȽԤ
	if (check_emap_down(EM_PIERCE, obj)){
        SetState(obj, os_pierced);
		return 0;
	}
    if (obj->u.creature.is_landed){
        // ⤱顢⤭³褦Ȥ
        if (IsWalking(obj)){
            //		if (!IsWalkBlocked(obj)){
			//  &&
//			(((obj->dir == di_left) && CheckEMap(Lbx(obj)-1, Lby(obj), eb_block_down)) ||
//			((obj->dir == di_right) && CheckEMap(Rbx(obj)+1, Rby(obj), eb_block_down))))
//			printf("yaramasi walk %d.\n", obj->u.creature.is_walking);
		}
        // äƤϤʤˤ⤷ʤ
        else if (obj->u.creature.is_sliding){
        }
        // Ϥ⤱ʤäϤ֤ĤäƤ뤫­줬ʤΤȿž
        else {
//            printf("turn %d\n", obj->dir);
            obj->dir = ReverseDir(obj->dir);
		}
		// ­줬ʤС⤫ʤ
		// 䤢餷ϼӹߤʤ
		if (obj->dir == di_left){
			if (CheckEMap(Lbx(obj)-1, Lby(obj), EM_BLOCK_DOWN)){
				SetMoveX(obj, -1);
			}
			else {
                SetWalkBlocked(obj, 1);
				SetMoveX(obj, 0);
			}
		}
		else {
			if (CheckEMap(Rbx(obj)+1, Rby(obj), EM_BLOCK_DOWN)){
				SetMoveX(obj, 1);
			}
			else {
                SetWalkBlocked(obj, 1);
				SetMoveX(obj, 0);
			}
		}
    }
    // 
    else {
        SetMoveX(obj, 0);
    }
//	obj->u.creature.move_x = 0; // !!!
	return 0;
}
void set_motion_yaramasi(obj_t *obj){
    switch (State(obj)){
	SkipDefaultCreatureMotions;
    default:
	    if (obj->u.creature.is_landed){
            SetWalkMotion(obj);
            /*
	        if (obj->dir == di_left){
		        obj->motion = (obj->motion == ii_yaramasi_walk_l0) ?
			        ii_yaramasi_walk_l1 : ii_yaramasi_walk_l0;
	        }
		    else if (obj->dir == di_right){
			    obj->motion = (obj->motion == ii_yaramasi_walk_r0) ?
				    ii_yaramasi_walk_r1 : ii_yaramasi_walk_r0;
	        }
            */
		}
		else {
            SetMotionDir(obj, ii_yaramasi_walk_l0, ii_yaramasi_walk_r0);
		}
        break;
    }
}
void set_emap_yaramasi(obj_t *obj){
    set_emap_creature(obj);
//    set_emap_box(Ltx(obj), Lty(obj)-2, obj->width, 2, eb_pierce);
    set_emap_box(Ltx(obj), Lty(obj), obj->width, 2, EM_PIERCE);
}

int set_action_gagga(obj_t *obj){
	if (!IsAlive(obj)) return 0;
	if (IsStunned(obj)) return 0;
	switch (State(obj)){
	case os_blast:
		IncCtr(obj);
		if (Ctr(obj) == GAGGA_HIT_COUNT){
			SetState(obj, os_none);
		}
		break;
	default:
        if (check_emap_box_any(Ltx(obj)-6-1, Lty(obj)+2, 6, obj->height-4,
                               EM_MEAT | EM_VEGETABLE | EM_WATER))
        {
			obj->dir = di_left;
			SetState(obj, os_blast);
		}
		else if (check_emap_box_any(Rtx(obj)+1+1, Rty(obj)+2, 6, obj->height-4,
                                    EM_MEAT | EM_VEGETABLE | EM_WATER))
        {
            obj->dir = di_right;
			SetState(obj, os_blast);
        }
		break;
	}
	return 0;
}
void set_motion_gagga(obj_t *obj){
    switch (State(obj)){
	SkipDefaultCreatureMotions;
    default:
		SetMotion(obj, ii_gagga_init);
		break;
	}
}

// creature¦eb_blastȽany_cornerǤäƤΤ
void set_emap_gagga(obj_t *obj){
    set_emap_creature(obj);
	if (State(obj) == os_blast && Ctr(obj) >= GAGGA_BLAST_COUNT){
		if (obj->dir == di_left){
		    set_emap_box(Ltx(obj)-4-1, Lty(obj)+1, 4, obj->height-1,
                         EM_BLAST_LEFT);
		    set_emap_box(Ltx(obj)-4-1, Lty(obj)+1, 4, obj->height-1, EM_SHOCK);
		}
		else {
		    set_emap_box(Rtx(obj)+1+1, Rty(obj)+1, 4, obj->height-1, 
                         EM_BLAST_RIGHT);
		    set_emap_box(Rtx(obj)+1+1, Rty(obj)+1, 4, obj->height-1, EM_SHOCK);
		}
	}
}
	
int set_action_nya(obj_t *obj){
	return 0;
}
void set_motion_nya(obj_t *obj){
}

int set_action_ojigiri(obj_t *obj){
    int v;
	obj_t *target;

	if (!IsAlive(obj)) return 0;
	if (IsStunned(obj)) return 0;
	IncCtr(obj);
	switch (State(obj)){
//	case os_cancel:
//        	SetState(obj, os_none);
//            break;
	case os_eating:
        if (Ctr(obj) == OJIGIRI_SWALLOW_COUNT){
			if (obj->dir == di_left){
		        v = check_emap_pos_box_any(Ltx(obj)-4, Rby(obj)-2, 2, 2,
		                                   EM_MEAT);
			}
			else {
				v = check_emap_pos_box_any(Rtx(obj)+3, Rby(obj)-2, 2, 2,
                                           EM_MEAT);
			}
			if (v){
				printf("eat target of %x:(%d, %d)\n",
				       v, CheckVal2PosX(v), CheckVal2PosY(v));
				target = GetEMapOwner(CheckVal2PosX(v), CheckVal2PosY(v), 
                                      _meat);
				assert(target != NULL);
				ReqState(target, obj, os_eaten);
			}
			else { // 
	        	SetState(obj, os_none);
			}
        }
        else if (Ctr(obj) >= OJIGIRI_EAT_COUNT){
			SetState(obj, os_digesting);
		}
		break;
	case os_digesting:
		if (Ctr(obj) >= OJIGIRI_DIGEST_COUNT){
			SetState(obj, os_vomitting);
		}
		break;
	case os_vomitting:
		if (Ctr(obj) >= OJIGIRI_VOMIT_COUNT){
			SetState(obj, os_none);
		}
		break;
	default:
		if (Ctr(obj) >= OJIGIRI_SWING_COUNT){
			SetCtr(obj, 0);
			obj->dir = ReverseDir(obj->dir);
		}
        if ((v = check_emap_pos_box_any(Ltx(obj)-4, Rby(obj)-2, 2, 2,
                                          EM_MEAT)))
        {
            obj->dir = di_left;
		    SetState(obj, os_eating);
        }
        else if ((v = check_emap_pos_box_any(Rtx(obj)+3, Rby(obj)-2, 2, 2,
                                             EM_MEAT)))
        {
            obj->dir = di_right;
		    SetState(obj, os_eating);
        }
		/*
		if (v){
//            printf("get target of %x:(%d, %d)\n",
//                   v, CheckVal2PosX(v), CheckVal2PosY(v));
//            obj->link = GetEMapOwner(CheckVal2PosX(v), CheckVal2PosY(v), eb_meat);
//            assert(obj->link != NULL);
		    SetState(obj, os_eating);
		}
			*/
		break;
    }
	return 0;
}

void set_motion_ojigiri(obj_t *obj){
	switch (State(obj)){
	SkipDefaultCreatureMotions;
	case os_none:
        SetMotionDir(obj, ii_ojigiri_l, ii_ojigiri_r);
		break;
	case os_digesting:
		obj->motion = (Ctr(obj) % 4 < 2) ?
            ii_ojigiri_digest0 : ii_ojigiri_digest1;
		break;
	default:
		break;
	}
}

int set_action_sue(obj_t *obj){
	if (!IsAlive(obj)) return 0;
	if (IsStunned(obj)) return 0;
	IncCtr(obj);
	switch (State(obj)){
    // blastedos_noneˤʤΤos_head_highѹƤ
	case os_none:
		SetState(obj, os_head_high);
		/* FALLTHRU */
	case os_head_high:
		if (Ctr(obj) >= SUE_WAIT_COUNT) SetState(obj, os_breath_high);
		break;
	case os_head_low:
		if (Ctr(obj) >= SUE_WAIT_COUNT) SetState(obj, os_breath_low);
		break;
	case os_breath_high:
		if (obj->dir == di_left){
				create_obj(ot_flame, Ltx(obj)-FLAME_WIDTH, Lty(obj), di_left);
		}
		else {
				create_obj(ot_flame, Rtx(obj)+1, Lty(obj), di_right);
		}
		if (Ctr(obj) >= SUE_BREATH_COUNT) SetState(obj, os_head_low);
		break;
	case os_breath_low:
		if (obj->dir == di_left){
				create_obj(ot_flame, Ltx(obj)-FLAME_WIDTH, Lty(obj)+4, di_left);
		}
		else {
				create_obj(ot_flame, Rtx(obj)+1, Lty(obj)+4, di_right);
		}
		if (Ctr(obj) >= SUE_BREATH_COUNT) SetState(obj, os_head_high);
		break;
    default:
        break;
	}
	return 0;
}
	
void set_motion_sue(obj_t *obj){
    switch (State(obj)){
	SkipDefaultCreatureMotions;
	case os_head_high:
        SetMotionDir(obj, ii_sue_high_close_l, ii_sue_high_close_r);
		break;
	case os_head_low:
        SetMotionDir(obj, ii_sue_low_close_l, ii_sue_low_close_r);
		break;
	case os_breath_high:
        SetMotionDir(obj, ii_sue_high_open_l, ii_sue_high_open_r);
		break;
	case os_breath_low:
        SetMotionDir(obj, ii_sue_low_open_l, ii_sue_low_open_r);
		break;
    default:
		break;
	}
}

int set_action_chu(obj_t *obj){
	obj_t *target;
	int v;

	if (!IsAlive(obj)) return 0;
	if (IsStunned(obj)) return 0;
	switch(State(obj)){
	case os_none:
		if (IsLanded(obj)){ // ˤȤ<->ΩΥȤߤ
			IncCtr(obj);
			if (IsWalkBlocked(obj)){
		        obj->dir = ReverseDir(obj->dir);
			}
            SetMoveXDir(obj);
            //SetMoveX(obj, (obj->dir == di_left) ? -1 : 1);
			if (Ctr(obj) == CHU_WALK_COUNT){
				SetState(obj, os_standing);
				obj->y--;
				obj->height = CHU_STAND_HEIGHT;
				SetMoveX(obj, 0);
			}
		}
        // 
        else {
            //            if (IsWalkBlocked(obj)){
            //                obj->dir = ReverseDir(obj->dir);
            //            }
            SetMoveX(obj, 0);
        }
		break;
	case os_standing:
		if (IsLanded(obj)){ // ˤȤ<->ΩΥȤߤ
			IncCtr(obj);
			if (Ctr(obj) == CHU_STAND_COUNT){
				SetState(obj, os_none);
				obj->y++;
				obj->height = CHU_WALK_HEIGHT;
				SetMoveX(obj, (obj->dir == di_left) ? -1 : 1);
			}
		}
		break;
	case os_cancel: // Ǥ򿩤٤Τ˼
		// ¨¤̾ư᤹
		SetCtr(obj, CHU_EAT_COUNT);
		/* FALLTHRU */
	case os_eating:
		IncCtr(obj);
        if (Ctr(obj) == CHU_EATING_COUNT){
			obj->height = CHU_WALK_HEIGHT;
			obj->y+= (GONDIN_HEIGHT-CHU_WALK_HEIGHT);
        }
        else if (Ctr(obj) >= CHU_EAT_COUNT){
            SetState(obj, os_none);
//			SetStunned(obj, 0);

			obj->x += 2; // ٤ƤȤγ32x32ʤΤǡٽäľγҤ褦˺ɸ򤺤餷Ƥ
			obj->width = CHU_WIDTH;
            obj->motion = (obj->dir == di_left) ? 
                ii_chu_stand_l0 : ii_chu_stand_r0; 
			SetMoveX(obj, (obj->dir == di_left) ? -1 : 1);
        }
		return 0;
	default:
		break;
	}
	if (//!IsStunned(obj) &&
        (v = check_emap_box_any_rim(Ltx(obj), Lty(obj)-1,
                                    obj->width, obj->height, EM_GONDIN)))
//         (v = check_emap_box_any(obj->x+3, obj->y+2, 1, 1, eb_gondin))))
    {
        printf("v=%d\n", v);
        target = GetEMapOwner(CheckVal2PosX(v), CheckVal2PosY(v), _gondin);
		obj->x = target->x;
		obj->y = target->y;
		obj->width = GONDIN_WIDTH;
		obj->height = GONDIN_HEIGHT;
        assert(target != NULL);
        ReqState(target, obj, os_gondin_eaten);
        puts("Gondin!!!");
        SetState(obj, os_eating);
        IncGameRecord(re_gondin_lost);
  		obj->u.creature.move_x = 0;
//        // ٤Ƥ֤ϽϤŨι뤬ưǽʤΤ
//        // stunnedˤƤ
//		SetStunned(obj, 1);
        obj->motion = ii_chu_eating0;
    }
	return 0;
}

/*
		if (Ctr(obj) % 2){
            obj->u.creature.move_x = (obj->dir == di_left) ? -1 : 1;
//			SetCtr(obj, 0);
		}
		else {
			obj->u.creature.move_x = 0;
		}
*/

void set_motion_chu(obj_t *obj){
    // ˴Υǥեȥѥʳ
    switch (State(obj)){
    case os_burned:
        switch (PeekState(obj)){
        case os_eating:
            SetMotionDir(obj, ii_chu_eating_burn_l, ii_chu_eating_burn_r);
            break;
        case os_standing:
            SetMotionDir(obj, ii_chu_stand_burn_l, ii_chu_stand_burn_r);
            break;
        default:
            // ԻΥ⡼ϥåȺѤ
            break;
        }
        break;
    case os_electrocuted:
        switch (PeekState(obj)){
        case os_eating:
            SetMotionDir(obj, ii_chu_stand_electrocute_l,
                         ii_chu_stand_electrocute_r); // !!!
            break;
        case os_standing:
            SetMotionDir(obj, ii_chu_stand_electrocute_l,
                         ii_chu_stand_electrocute_r);
            break;
        default:
            // ԻΥ⡼ϥåȺѤ
            break;
        }
        break;
    default:
        break;
    }
    switch (State(obj)){
	SkipDefaultCreatureMotions;
	case os_standing:
        if (obj->dir == di_left){
            obj->motion = (obj->motion == ii_chu_stand_l0) ?
                ii_chu_stand_l1 : ii_chu_stand_l0;
        }
        else if (obj->dir == di_right){
            obj->motion = (obj->motion == ii_chu_stand_r0) ?
                ii_chu_stand_r1 : ii_chu_stand_r0;
        }
		break;
    default:
/*
		if (obj->u.creature.is_sliding){
            obj->motion = (obj->dir ==  di_left) ? ii_hatopo_sl : ii_hatopo_sr;
        }
		else if (IsStunned(obj)){
            obj->motion = (obj->dir ==  di_left) ? ii_hatopo_bl : ii_hatopo_br;
		}
*/
        // 
        if (!obj->u.creature.is_landed){
            SetMotionDir(obj, ii_chu_walk_l0, ii_chu_walk_r0);
        }
        // Ի
        else {
            SetWalkMotion(obj);
            /*
            if (obj->dir == di_left){
                obj->motion = (obj->motion == ii_chu_walk_l0) ?
                    ii_chu_walk_l1 : ii_chu_walk_l0;
            }
            else if (obj->dir == di_right){
                obj->motion = (obj->motion == ii_chu_walk_r0) ?
                    ii_chu_walk_r1 : ii_chu_walk_r0;
            }
            */
        }
        break;
    }
}
int set_action_mingo(obj_t *obj){
    int attack;
	if (!IsAlive(obj)) return 0;
	if (IsStunned(obj)) return 0;
	// ɤᡢat_immune_pierceˤƤơ
	// ܤ򳰤ȽԤ
    // Τᡢߤ󤴤ϲˤϤĤäޤȿžƲ򤹤롣
    // ϻͤȤ?
    //    x = (obj->dir == di_left) ? Ltx(obj)+2 : Ltx(obj);
    //	if (check_emap_box_any_rim(x, Lty(obj), obj->width-2, obj->height, 
    //                               EM_PIERCE))
    // slopeǹ⤷ʤ麸˰ưǽΤǡϰϤ餵˰
    // Ƚꤹ
    if (check_emap_box_any_rim_rel(obj, 3, 0, obj->width-1, obj->height-1,
                                   EM_PIERCE))
    {
        SetState(obj, os_pierced);
		return 0;
	}

    switch (State(obj)){
    case os_none:
        //        x = (obj->dir == di_left) ? Ltx(obj)-1 : Rtx(obj)-2;
        //        x = (obj->dir == di_left) ? Ltx(obj)-1 : Rtx(obj)+1;
        attack = check_emap_box_any_rim_rel(obj, -1, 0, 2, obj->height-1,
                                            EM_CHEE) ||
            check_emap_rel(obj, -1, 0, EM_FWALL);
        //        attack = check_emap_box_any_rim(x, Lty(obj),
        //                                        4, obj->height, EM_CHEE) ||
        if (attack){
            SetState(obj, os_mingo_attack);
            return 0;
        }
        break;
    case os_mingo_attack:
        IncCtr(obj);
        if (Ctr(obj) > MINGO_ATTACK_COUNT) SetState(obj, os_none);
        break;
    default:
        break;
    }

    if (IsLanded(obj)){
        // ⤱餽Τޤ⤭³
        if (!IsWalkBlocked(obj)){
            SetMoveXDir(obj);
            printf("continue walking %d\n", MoveX(obj));
        }
        // äƤϤʤˤ⤷ʤ
        else if (IsSliding(obj)){
        }
        // ʤˤ⤷ʤ
        else if (State(obj) == os_mingo_attack){
        }
        // Ϥ⤱ʤϤ֤ĤäƤΤȿž
        else {
            //            printf("turn %d\n", obj->dir);
            obj->dir = ReverseDir(obj->dir);
            SetMoveXDir(obj);
        }
    }
    else {
        if (IsWalkBlocked(obj)){
            obj->dir = ReverseDir(obj->dir);
        }
        SetMoveX(obj, 0);
    }
	return 0;
}
void set_motion_mingo(obj_t *obj){
    switch (State(obj)){
	SkipDefaultCreatureMotions;
    case os_mingo_attack:
        if (obj->dir == di_left){
            obj->motion = (Ctr(obj) % 2) ?
                ii_mingo_attack_l0 : ii_mingo_attack_l1;
        }
        else {
            obj->motion = (Ctr(obj) % 2) ?
                ii_mingo_attack_r0 : ii_mingo_attack_r1;
        }
        break;
    default:
        if (obj->u.creature.is_sliding){
            SetMotionDir(obj, ii_mingo_slide_l, ii_mingo_slide_r);
        }
        // ­򳫤ޤ޸
        else if (!obj->u.creature.is_landed){
            SetMotionDir(obj, ii_mingo_slide_l, ii_mingo_slide_r);
        }
        // Ի
        // @note 2ޥ˥SetWalkMotionϻȤʤΤǡľ
        else {
            if (obj->dir == di_left){
                obj->motion ++;
                if (obj->motion > ii_mingo_walk_l3 || 
                    obj->motion < ii_mingo_walk_l0)
                {
                    obj->motion = ii_mingo_walk_l0;
                }
            }
            else if (obj->dir == di_right){
                obj->motion ++;
                if (obj->motion > ii_mingo_walk_r3 || 
                    obj->motion < ii_mingo_walk_r0)
                {
                    obj->motion = ii_mingo_walk_r0;
                }
            }
        }
        break;
    }
}

void set_emap_mingo(obj_t *obj){
    set_emap_creature(obj);
    if (State(obj) == os_mingo_attack){
        set_emap_box_rel(obj, 0, 1, 1, 3, EM_PIERCE);
        // fwall˲Τshock
        //        if (check_emap_rel(obj, -1, 0, EM_FWALL)){
        set_emap_rel(obj, -1, 0, EM_SHOCK);
        set_emap_rel(obj, -1, GRID_SIZE, EM_SHOCK);
        /*
        if (obj->dir == di_left){
            set_emap_box(Ltx(obj), Lty(obj)+1, 1, 3, EM_PIERCE);
            //            if (check_emap_rel(obj, -1, 0, EM_FWALL);
        }
        else {
            set_emap_box(Rtx(obj), Lty(obj)+1, 1, 3, EM_PIERCE);
        }
        */
    }
}

int set_action_jumeme(obj_t *obj){
	if (!IsAlive(obj)) return 0;
	if (IsStunned(obj)) return 0;
    if (IsLanded(obj)){	
        if (check_emap_box_any(Ltx(obj)+2, Lty(obj)-1, obj->width-4,
                                1, EM_ICE)){
            SetState(obj, os_jumeme_melting_upper);
			SetMoveX(obj, 0);
            return 0;
        }
//        if (obj->dir == di_left && check_emap_box_any(Ltx(obj)-1, Lty(obj), 1, obj->height, eb_ice)){
        if (check_emap_box_any(Ltx(obj)-1, Lty(obj), 1, obj->height, EM_ICE)){
            obj->dir = di_left; 
			SetState(obj, os_jumeme_melting_side);
			SetMoveX(obj, 0);
            return 0;
        }
//        if (obj->dir == di_right && check_emap_box_any(Rtx(obj)+1, Lty(obj), 1, obj->height, eb_ice)){
        if (check_emap_box_any(Rtx(obj)+1, Lty(obj), 1, obj->height, EM_ICE)){
            obj->dir = di_right;
			SetState(obj, os_jumeme_melting_side);
			SetMoveX(obj, 0);
            return 0;
        }
        if (check_emap_box_any(Ltx(obj)+2, Lby(obj)+1, obj->width-4,
                                1, EM_ICE)){
            SetState(obj, os_jumeme_melting_lower);
			SetMoveX(obj, 0);
            return 0;
        }
        SetState(obj, os_none);
		SetMoveXDir(obj);
        // ⤱餽Τޤ⤭³
        if (!IsWalkBlocked(obj)){
            printf("continue walking %d\n", MoveX(obj));
        }
        // äƤϤʤˤ⤷ʤ
        else if (obj->u.creature.is_sliding){
        }
        // Ϥ⤱ʤϤ֤ĤäƤΤȿž
        else {
            //            printf("turn %d\n", obj->dir);
            obj->dir = ReverseDir(obj->dir);
            SetMoveXDir(obj);
        }
    }
    else { // 
        if (IsWalkBlocked(obj)){
            obj->dir = ReverseDir(obj->dir);
        }
        if (obj->dir == di_left){
            if (CheckEMap(Lbx(obj)-1, Lby(obj)+0, EM_BLOCK_DOWN) ||
                CheckEMap(Lbx(obj)-1, Lby(obj)+1, EM_BLOCK_DOWN))
                //                CheckEMap(Lbx(obj)-1, Lby(obj)+1, EM_EXCLUDE_UP))
            {
                SetMoveXDir(obj);
            }
            else {
                SetMoveX(obj, 0);
            }
        }
        else {
            if (CheckEMap(Rbx(obj)+1, Rby(obj)+0, EM_BLOCK_DOWN) ||
                CheckEMap(Rbx(obj)+1, Rby(obj)+1, EM_BLOCK_DOWN))
            {
                SetMoveXDir(obj);
            }
            else {
                SetMoveX(obj, 0);
            }
        }
    }
	return 0;
}
void set_motion_jumeme(obj_t *obj){
    switch (State(obj)){
	SkipDefaultCreatureMotions;
	case os_jumeme_melting_upper:
		if (obj->dir ==  di_left){
			obj->motion = (obj->motion == ii_jumeme_melt_ul0) ?
                ii_jumeme_melt_ul1 : ii_jumeme_melt_ul0;
		}
		else {
			obj->motion = (obj->motion == ii_jumeme_melt_ur0) ?
                ii_jumeme_melt_ur1 : ii_jumeme_melt_ur0;
		}
        break;
	case os_jumeme_melting_side:
		if (obj->dir ==  di_left){
			obj->motion = (obj->motion == ii_jumeme_melt_l0) ?
                ii_jumeme_melt_l1 : ii_jumeme_melt_l0;
		}
		else {
			obj->motion = (obj->motion == ii_jumeme_melt_r0) ?
                ii_jumeme_melt_r1 : ii_jumeme_melt_r0;
		}
        break;
	case os_jumeme_melting_lower:
		obj->motion = (obj->motion == ii_jumeme_melt_sit0) ? 
            ii_jumeme_melt_sit1 : ii_jumeme_melt_sit0;
        break;
    default:
        if (obj->u.creature.is_sliding){
            obj->motion = (obj->dir ==  di_left) ? 
                ii_jumeme_walk_l0 : ii_jumeme_walk_r0;
        }
		// ­
        // !! ii_jumeme_slide_l/rϥ֥å˾ܤȤ­褬ŤʤΤǻȤʤ
        else if (!obj->u.creature.is_landed){
            SetMotionDir(obj, ii_jumeme_walk_l0, ii_jumeme_walk_r0);
        }
        // Ի
        else {
            SetWalkMotion(obj);
            /*
            if (obj->dir == di_left){
                obj->motion = (obj->motion == ii_jumeme_walk_l0) ?
                    ii_jumeme_walk_l1 : ii_jumeme_walk_l0;
            }
            else if (obj->dir == di_right){
                obj->motion = (obj->motion == ii_jumeme_walk_r0) ?
                    ii_jumeme_walk_r1 : ii_jumeme_walk_r0;
            }
            */
        }
        break;
    }
}

void set_emap_jumeme(obj_t *obj){
    set_emap_creature(obj);
    if (!IsAlive(obj)) return;
    switch (State(obj)){
    case os_jumeme_melting_upper:
        set_emap_box(obj->x, obj->y-1, obj->width, obj->height+1, EM_FLAME);
        break;
    case os_jumeme_melting_side:
		if (obj->dir == di_left){
	        set_emap_box(obj->x-1, obj->y, obj->width+1, obj->height, EM_FLAME);
		}
		else {
        set_emap_box(obj->x, obj->y, obj->width+1, obj->height, EM_FLAME);
		}
			break;
    case os_jumeme_melting_lower:
        set_emap_box(obj->x, obj->y+1, obj->width, obj->height, EM_FLAME);
        break;
    default:
        set_emap_box(obj->x, obj->y, obj->width, obj->height, EM_FLAME);
        break;
    }
}

int set_action_siguku(obj_t *obj){
    switch (State(obj)){
    case os_none:
        if (check_creature_landed(obj) && !IsSliding(obj)){
            printf("drop landed\n");
            SetState(obj, os_siguku_landed);
        }
        break;
    case os_siguku_landed:
        if (IsSliding(obj)){
	        SetState(obj, os_none);
            break;
        }
        if (check_emap_box_any(Ltx(obj), Rby(obj)+1, obj->width, 1, EM_ICE)){
            printf("create ice\n");
            SetState(obj, os_siguku_freezing);
            return 0;
        }
        else if (check_creature_landed(obj)){
	        SetState(obj, os_siguku_splashing);
			break;
        }
        SetState(obj, os_none);
		break;
    case os_siguku_splashing:
		// ̤ƾäϻ˴Ȥʤ
//		SetState(obj, os_dead);
		return 1;
    case os_siguku_freezing:
		IncCtr(obj);
		if (Ctr(obj) >= FREEZE_COUNT){
	        printf("CREATE ICE\n");
            // ߺɸ˶ᤤåɰ֤iwall
            // @note åɤ4ɸñ̹ߤʤΤǡ+2ƤڤΤƤ뤳ȤǰƤ
		    create_obj(ot_iwall, (obj->x+2) & 0xfffc, (obj->y+2) & 0xfffc, di_none);
		// ϻ˴Ȥʤ
            return 1;
		}
		break;
    default:
        break;
    }
    return 0;
}
void set_motion_siguku(obj_t *obj){
    if (BlastX(obj)){
        obj->dir = (BlastX(obj) < 0) ? di_left : di_right;
    }
    switch (State(obj)){
	SkipDefaultCreatureMotions;
	case os_siguku_freezing:
        SetMotion(obj, ii_siguku_freezing);
        break;
	case os_siguku_splashing:
        SetMotion(obj, ii_siguku_splash);
        break;
    default:
        SetMotion(obj, ii_siguku);
        break;
    }
}
void set_emap_siguku(obj_t *obj){
    set_emap_creature(obj);
    set_emap_box(obj->x, obj->y, obj->width, obj->height, EM_WATER);
}

int act_nya(obj_t *obj){
    obj_t *punch = obj->link;

    switch (State(obj)){
    case os_req_off:
        SetState(obj, os_nya_halting);
        punch->dir = ReverseDir(obj->dir);
        SetMotionDir(obj, ii_nya_halting_l, ii_nya_halting_r);
        break;
    case os_req_on:
        SetState(obj, os_on);
        SetMotionDir(obj, ii_nya_on_l, ii_nya_on_r);
        // halting/offǥѥǼѤ/ǼΤȤΤߡ
        // ѥθ֤
        if (Rtx(punch) > Ltx(obj) || Rtx(obj) < Ltx(punch)){
            punch->dir = obj->dir;
            punch->x = (obj->dir == di_left) ?
            PunchInitPosLeft(obj) : PunchInitPosRight(obj);
        }
        break;
    case os_nya_halting:
    case os_off:
    case os_on:
        break;
    default:
        break;
    }
    return 0;
}		

int setup_nya(obj_t *obj){
    obj_t *punch = obj->link;

    printf("setup nya %d: %d %d %d\n", State(obj), os_on, os_off, os_nya_halting);
    switch (State(obj)){
    case os_req_off:
    case os_off:
        SetState(obj, os_off);
        SetMotionDir(obj, ii_nya_off_l, ii_nya_off_r);
        punch->x = obj->x;
        break;
    case os_req_on:
    case os_on:
        SetState(obj, os_on);
        SetMotionDir(obj, ii_nya_on_l, ii_nya_on_r);
        punch->dir = obj->dir;
        punch->x = (obj->dir == di_left) ?
            PunchInitPosLeft(obj) : PunchInitPosRight(obj);
        break;
    case os_nya_halting:
    default:
        break;
    }

	set_emap_punch(obj->link);
	return 0;
}
void set_emap_punch(obj_t *obj){
	set_emap_blockbox(Ltx(obj), Lty(obj), obj->width, obj->height);
}

int act_punch(obj_t *obj){
    //	int turn = 0;
    obj_t *nya = obj->link;
    
    if (State(nya) == os_off) return 0;
    switch (nya->dir){
    case di_left:
        switch(obj->dir){
        case di_left:
            if (check_emap_obj_any_left(obj, EM_BLOCK_LEFT)){
                set_emap_box(Ltx(obj)-1, Lty(obj), 4, obj->height, EM_SHOCK);
                obj->dir = di_right;
                //                turn = 1;
            }
            else {
                obj->x --;
				set_emap_box(Ltx(obj), Lty(obj)-1, 
                             Ltx(nya)-Ltx(obj)+obj->width, 1, 
                             EM_PUSH_LEFT);
                set_emap_box(Ltx(obj)-1, Lty(obj), 2, obj->height, EM_PIERCE);
            }
            break;
        case di_right:
            if ((Rtx(obj) < Ltx(nya)-2 &&
                 check_emap_obj_any_right(obj, EM_BLOCK_RIGHT)) ||
                (Rtx(obj) == Ltx(nya)+0 &&
                 State(nya) != os_nya_halting))
            {
                obj->dir = di_left;
                //                turn = 1;
            }
            else if (Ltx(obj) == Ltx(nya) ){
                SetMotionDir(nya, ii_nya_off_l, ii_nya_off_r);
            }
            else {
                obj->x ++;
				set_emap_box(Ltx(obj), Lty(obj)-1, 
                             Ltx(nya)-Ltx(obj)+obj->width, 1, 
                             EM_PUSH_RIGHT);
            }
            break;
        default:
            assert(0);
        }
        if (Rtx(obj) < Ltx(nya)){
            set_emap_blockbox(Rbx(obj)+2, Lty(obj), Ltx(nya)-Ltx(obj)-2, 2);
        }
        break;
    case di_right:
        switch(obj->dir){
        case di_right:
            if (check_emap_obj_any_right(obj, EM_BLOCK_RIGHT)){
                set_emap_box(Rbx(obj)+1, Lty(obj), 4, obj->height, EM_SHOCK);
                obj->dir = di_left;
                //                turn = 1;
            }
            else {
                obj->x ++;
				set_emap_box(Ltx(nya), Lty(obj)-1,
                             Ltx(obj)-Ltx(nya)+obj->width, 1, EM_PUSH_RIGHT);
                set_emap_box(Rtx(obj)-1+1, Lty(obj), 2, obj->height, EM_PIERCE);
            }
            break;
        case di_left:
            if ((Ltx(obj) > Rtx(nya)+2 &&
                 check_emap_obj_any_left(obj, EM_BLOCK_LEFT)) ||
                (Ltx(obj) == Rtx(nya)+0 &&
                 State(nya) != os_nya_halting))
            {
                obj->dir = di_right;
                //                turn = 1;
            }
            else if (Ltx(obj) == Ltx(nya) ){
                SetState(nya, os_off);
                SetMotionDir(nya, ii_nya_off_l, ii_nya_off_r);
            }
            else {
                obj->x --;
				set_emap_box(Ltx(nya), Lty(obj)-1,
                             Ltx(obj)-Ltx(nya)+obj->width, 1, EM_PUSH_LEFT);
            }
            break;
        default:
            assert(0);
        }
        if (Ltx(obj) > Rtx(nya)){
            set_emap_blockbox(Ltx(nya), Lty(obj), Ltx(obj)-Ltx(nya)-2, 2);
        }
        break;
    default:
        break;
    }
    set_emap_punch(obj);
	return 0;



/*
			}
		}
		else {
			if (!turn){
				set_emap_box(Ltx(obj), Lty(obj)-1,
                             Ltx(obj->link)-Ltx(obj)+obj->width, 1,
                             EM_PUSH_RIGHT);
			}
		}


    case di_right:
        if (!check_emap_dir(EM_BLOCK_RIGHT,di_right, obj)){
            obj->x ++;
        }         
        else {
            set_emap_box(Rbx(obj)+1, obj->y, 4, obj->height, EM_SHOCK);
            obj->dir = di_left;
			turn = 1;
        }
        break;
    default:
        break;
    }
    if (nya->dir == di_left){
		if (obj->x == nya->x-2){
	        obj->dir = ReverseDir(obj->dir);
			turn = 1;
            if (State(nya) == os_nya_halting){
                ReqState(nya, obj, os_off);
            }
		}
	}
	else {
		if (obj->x == obj->link->x+6){
	        obj->dir = ReverseDir(obj->dir);
			turn = 1;
            if (State(nya) == os_nya_halting){
                ReqState(nya, obj, os_off);
			}
		}
	    set_emap_blockbox(Ltx(obj->link), Lty(obj), Ltx(obj)-Ltx(obj->link)-2, 2);
		if (obj->dir == di_right){
			set_emap_box(Rtx(obj)-1+1, Lty(obj), 2, obj->height, EM_PIERCE);
			if (!turn){
				set_emap_box(Ltx(obj->link), Lty(obj)-1, Ltx(obj)-Ltx(obj->link)+obj->width, 1, EM_PUSH_RIGHT);
			}
		}
		else {
			if (!turn){
				set_emap_box(Ltx(obj->link), Lty(obj)-1, Ltx(obj)-Ltx(obj->link)+obj->width, 1, EM_PUSH_LEFT);
			}
		}
	}
*/
}


int act_lift(obj_t *obj){
    //    printf("lift state:%d prev state:%d\n", State(obj), PrevState(obj));
    switch (State(obj)){
    case os_req_off:
        SetState(obj, os_off);
        break;
    case os_req_on:
        SetState(obj, os_on);
        break;
    case os_off:
        while (check_emap_obj_any_down(obj, EM_EXCLUDE_UP)){
            obj->y --;
            //            printf("lift exclude up %d\n", obj->y);
        }
        /*
        while (check_emap_obj_any_up(obj, eb_exclude_down)){
            obj->y ++;
            printf("lift exclude down %d\n", obj->y);
        }
        */
        break;
    case os_on:
	    switch (obj->dir){
	    case di_up:
		    if (!check_emap_dir(EM_BLOCK_UP|EM_EXCLUDE_DOWN, di_up, obj)){
			    obj->y --;
	            set_emap_box(Ltx(obj), Lty(obj)-1, obj->width, 1, EM_PUSH_UP);
	        }
	        else {
		        set_emap_box(Ltx(obj), Lty(obj)-1, obj->width, 4, EM_SHOCK);
				obj->dir = di_down;
	            //            puts ("switch u->d");
		    }
			break;
	    case di_down:
		    if (!check_emap_dir(EM_BLOCK_DOWN|EM_EXCLUDE_UP, di_down, obj)){
			    obj->y ++;
                //                printf("set eb_push_down\n");
	            set_emap_box(Ltx(obj), Lty(obj)-1, obj->width, 1, EM_PUSH_DOWN);
                //	            set_emap_box(Lbx(obj), Lby(obj), obj->width, 1, eb_weight);
                set_emap_owner_box(Lbx(obj), Lby(obj), obj->width, 1,
                                   obj, en_weight);
	        }
	        else {
	            set_emap_box(Ltx(obj), Rby(obj)+1, obj->width, 4, EM_SHOCK);
                /*
				// ξ㳲ʪˤ֤Ĥä֤ǻߤޤäƤƤweightȯ
				set_emap_box(Lbx(obj), Lby(obj), obj->width, 1, eb_weight);
                set_emap_owner_box(Lbx(obj), Lby(obj), obj->width, 1,
                                   eb_weight, obj);
                */
			    obj->dir = di_up;
				//            puts ("switch u->d");
	        }
		    break;
	    case di_left:
		    //        printf("(%d, %d) %dx%d\n", obj->x, obj->y, obj->width, obj->height);
			if (!check_emap_dir(EM_BLOCK_LEFT|EM_EXCLUDE_RIGHT, di_left, obj)){
				obj->x --;
	            set_emap_box(Ltx(obj), Lty(obj)-1, obj->width, 1,
                             EM_PUSH_LEFT);
			}
	        else {
		        set_emap_box(Ltx(obj)-1, Lty(obj), 4, obj->height, EM_SHOCK);
				obj->dir = di_right;
	            //            puts ("switch l->r");
		    }
			break;
	    case di_right:
		    if (!check_emap_dir(EM_BLOCK_RIGHT|EM_EXCLUDE_LEFT, di_right, obj)){
			    obj->x ++;
	            set_emap_box(Ltx(obj), Lty(obj)-1, obj->width, 1,
                             EM_PUSH_RIGHT);
	        }         
		    else {
			    set_emap_box(Rbx(obj)+1, obj->y, 4, obj->height, EM_SHOCK);
	            obj->dir = di_left;
		        //            puts ("switch r->l");
			}
	        break;
		default:
			break;
	    }
        break;
    default:
        break;
	}
	set_emap_lift(obj);
/*
	// ξ㳲ʪˤ֤Ĥä֤ǻߤޤäƤƤweightȯ
    set_emap_box(Lbx(obj), Lby(obj), obj->width, 1, eb_weight);
    set_emap_owner_box(Lbx(obj), Lby(obj), obj->width, 1, eb_weight, obj);
	*/
    SavePrevState(obj);
    return 0;
}

int setup_lift(obj_t *obj){
    printf("%p lift state=%d (%d %d)\n", obj, State(obj), os_on, os_off);
	set_emap_lift(obj);
	return 0;
}
void set_emap_lift(obj_t *obj){
    set_emap_blockbox(obj->x, obj->y, obj->width, obj->height);
}



int act_mswitch(obj_t *obj){
    // !!! PrevState˺ؤ줽
    obj_state_t old_state = State(obj);

    if (check_emap_box_any(Ltx(obj), Rty(obj)-1, obj->width, 1, EM_WEIGHT)){
        //		puts("mswitch weighted");
        if (obj->u.mswitch.is_reverse){
            SetState(obj, os_off);
            obj->motion = ii_mswitch_off;
        }
        else {
            SetState(obj, os_on);
            obj->motion = ii_mswitch_on;
        }
	}
	else {
        if (obj->u.mswitch.is_reverse){
            SetState(obj, os_on);
            obj->motion = ii_mswitch_on;
        }
        else {
            SetState(obj, os_off);
            obj->motion = ii_mswitch_off;
        }
	}
    if (State(obj) != old_state){
    //	if (State(obj) != PrevState(obj)){
		if (obj->link){
			IncGameRecord(re_switch_count);
            printf("mswitch reqstate %d\n", State(obj));
			ReqState(obj->link, obj, State(obj) == os_on ? os_req_on : os_req_off);
		}
    }
    SavePrevState(obj);
	return 0;
}

int setup_mswitch(obj_t *obj){
    return act_mswitch(obj);
}

void on_aswitch(obj_t *obj){
    IncGameRecord(re_switch_count);
    obj->motion = ii_aswitch_on;
    obj->y++;
    obj->height --;
    if (obj->link){
        ReqState(obj->link, obj, os_req_on);
    }
}

void off_aswitch(obj_t *obj){
    IncGameRecord(re_switch_count);
    obj->motion = ii_aswitch_off;
    obj->y--;
    obj->height ++;
    if (obj->link){
        ReqState(obj->link, obj, os_req_off);
    }
    printf("nwon->woff y=%d\n", obj->y);
}

// aswitch˽ŤߤݤƤobjФΥݥ󥿤֤
// оݤhard/softǸƤӽФ¦νۤʤ١
// ¸ߤ̵ͭǤʤobjؤΥݥ󥿤֤obj->type
// ȤǤ褦ˤ
obj_t *check_aswitch_weight(obj_t *obj, int y){
	int x=0, lx, rx;
	obj_t *target_old = NULL, *target;

	for (x = Ltx(obj) ; x <= Rtx(obj) ; x++){
	    if (CheckEMap(x, y, EM_WEIGHT)){
            target = GetEMapOwner(x, y, _weight);
	        if (target != NULL && target != target_old){
                printf("(%d, %d) found weight:(%d,%d)\n", x, y, target->x, target->y);
                target_old = target;
                if (IsCreature(target)){
                    lx = LandLpx(target);
                    rx = LandRpx(target);
                }
                else {
                    lx = Lbpx(target);
                    rx = Rbpx(target);
                }
//				if ((!CheckEMap(Ltx(obj)-1, Lty(obj), eb_exclude_up) ||
//				if ((!CheckEMap(Ltx(obj)-1, Lty(obj)-1, EM_BLOCK_DOWN) ||
				if ((!CheckEMap(Ltx(obj)-1, y, EM_BLOCK_DOWN) ||
                     Ltx(obj) <= lx) && 
//					(!CheckEMap(Rtx(obj)+1, Rty(obj), eb_exclude_up) || 
//					(!CheckEMap(Rtx(obj)+1, Rty(obj)-1, EM_BLOCK_DOWN) || 
					(!CheckEMap(Rtx(obj)+1, y, EM_BLOCK_DOWN) || 
                     rx <= Rtx(obj)))
				{
					return target;
				}
            }
		}
	}
	return NULL;
}

int act_aswitch(obj_t *obj){
	int y;
	obj_t *target;
    // hard_dynamicsoft_dynamicη̤next_emap˽񤤤Ƥ
    // äơcreature¸߾Ϥ1ƥåΤΤˤʤ
	// ΤᡢON/OFFڤؤ1frame־(xx_int)ߤ
	// Ⱦüʾȿʤ褦ˤ

    //    printf("act aswitch %d\n", State(obj));
	switch(State(obj)){
	case os_req_on:
		on_aswitch(obj);
        SetState(obj, os_weighted_on_int);
		break;
	case os_req_off:
		off_aswitch(obj);
        SetState(obj, os_weighted_off_int);
		break;
	case os_weighted_on:
    	y = Lty(obj)-1;
		if (!check_aswitch_weight(obj, y)){
            //		    puts("found won->nwon!");
	        SetState(obj, os_nonweighted_on);
//			change_aswitch_weighted(obj);
		}
		else{
            //	        puts("still weighted!");
		}
 		break;
	case os_weighted_off:
    	y = Lty(obj)-1;
		if (!check_aswitch_weight(obj, y)){
            //		    puts("found woff->nwoff!");
	        SetState(obj, os_nonweighted_off);
//			change_aswitch_weighted(obj);
		}
		else{
            //	        puts("still weighted!");
		}
		break;
	case os_nonweighted_off:
    	y = Lty(obj)-1;
		if (check_aswitch_weight(obj, y)){
            //		    puts("found off->on!");
			on_aswitch(obj);
	        SetState(obj, os_weighted_on_int);
//			change_aswitch_weighted(obj);
		}
		else{ // å˴˾äƤԤʤä
            //	        puts("not found on!");
		}
 		break;
	case os_weighted_on_int:
        SetState(obj, os_weighted_on);
		break;
	case os_nonweighted_on:
    	y = Lty(obj)-1;
		if ((target = check_aswitch_weight(obj, y))){
            //		    puts("found on->off!");
			if (target->type == ot_lift){
				SetState(obj, os_req_off);
			}
			else {
				off_aswitch(obj);
		        SetState(obj, os_weighted_off_int);
			}
//			change_aswitch_weighted(obj);
		}
		else{ // å˴˾äƤԤʤä
            //	        puts("not found off!");
		}
 		break;
	case os_weighted_off_int:
        SetState(obj, os_weighted_off);
		break;
	default:
		break;

	}
	set_emap_aswitch(obj);
    SavePrevState(obj);
	return 0;
}

int setup_aswitch(obj_t *obj){
    obj_t *target;

    switch (State(obj)){
    case os_off:
    case os_weighted_off:
    case os_nonweighted_off:
        if (check_aswitch_weight(obj, Lty(obj)-1)){
            SetState(obj, os_weighted_off);
            //            printf("WEIGHTED_OFF!\n");
        }
        else {
            SetState(obj, os_nonweighted_off);
            //            printf("NONWEIGHTED_OFF!\n");
        }
        break;
    // +ץǽONˤȤ
    case os_req_on:
        printf("setup os_req_on\n");
        on_aswitch(obj);
        /* FALLTHRU */
    case os_nonweighted_on:
        printf("setup os_nonweighted_on\n");
        if ((target = check_aswitch_weight(obj, Lty(obj)-2))){
            target->y ++; // äƤcreature򥹥åι⤵˹碌Ʋ
            SetState(obj, os_weighted_on);
                       printf("WEIGHTED_ON!\n");
        }
        else {
            SetState(obj, os_nonweighted_on);
                        printf("NONWEIGHTED_ON!\n");
        }
        break;
    default:
        //        printf("ELSE!\n");
        break;
    }
    //	act_aswitch(obj);
	set_emap_aswitch(obj);
	return 0;
}
void set_emap_aswitch(obj_t *obj){
    set_emap_blockbox(obj->x, obj->y, obj->width, obj->height);
}

int act_teleporter(obj_t *obj){
    //    printf("teleporter state:%d\n", State(obj));
    switch (State(obj)){
    case os_req_on:
        SetState(obj, os_on);
        break;
    case os_req_off:
        SetState(obj, os_off);
        break;
    default:
        break;
    }
    set_motion_teleporter(obj);
    set_emap_teleporter(obj);
    return 0;
}

int setup_teleporter(obj_t *obj){
    set_motion_teleporter(obj);
	// setup˥ƥݡȾɬפʤϤ
    // set_emap_teleporter()ϲξ㳲ʪǻߤޤޤ
	// ƥݡȾꤹΤ
	// ƥݡ겼ξ㳲ʪemapꤹ˸Ƥ֤
	// ϰϳޤꤷƤޤ
	//	set_emap_teleporter(obj);
	return 0;
}

void set_motion_teleporter(obj_t *obj){
    if (State(obj) == os_on){
        SetMotion(obj, ii_teleporter_on);
    }
    else {
        SetMotion(obj, ii_teleporter_off);
    }
}

void set_emap_teleporter(obj_t *obj){
    int x, y, height;
    if (State(obj) == os_on){
        for (x = Ltx(obj) ; x < Rtx(obj) ; x += GRID_SIZE){
            y = Lby(obj)+1;
            height = 0;
			assert(x >= MAP_LTX);
			assert(y >= MAP_LTY);
			assert(x <= MAP_RBX);
			assert(y <= MAP_RBY);
            while (!CheckEMap(x, y, EM_EXCLUDE_UP)){
                height += GRID_SIZE;
                y += GRID_SIZE;
				assert(y <= MAP_RBY);
            }
            set_emap_owner_box(x, Lby(obj)+1, GRID_SIZE, height, 
                               obj, en_teleport);
        }
        //        set_emap_box(Ltx(obj), Lty(obj), obj->width, obj->height,
        //                     eb_teleport, obj);
        //        set_emap_owner_box(Ltx(obj), Lby(obj), obj->width, obj->height,
        //                           eb_teleport, obj);
    }
}

int act_goal(obj_t *obj){
	switch (State(obj)){
    case os_none:
		break;
    case os_goal_req_open:
        if (obj->motion == ii_goal_closed){
            SetState(obj, os_goal_opening);
        }
        else {
            SetState(obj, os_goal_opened);
        }
        break;
	case os_goal_opening:
		IncCtr(obj);
		switch (Ctr(obj)){
		case 1:
			obj->motion = ii_goal_open1;
			break;
		case 2:
			obj->motion = ii_goal_open2;
			break;
		case 3:
			obj->motion = ii_goal_opened;
            SetState(obj, os_goal_opened);
            // ٳиϥꥢޤĤʤ顢staticꤷƤ褤
            set_emap_static_box(obj->x, obj->y, obj->width, obj->height, 
                                EM_GOAL);
			break;
		default:
			assert(0);
			break;
		}
		break;
	case os_goal_opened:
		break;
	case os_exit:
		IncCtr(obj);
        if (Ctr(obj) == CHEE_EXIT_COUNT){
            SetState(obj, os_goal_closing);
        }
		break;
	case os_goal_closing:
		IncCtr(obj);
		switch (Ctr(obj)){
		case 1:
			obj->motion = ii_goal_open2;
			break;
		case 2:
			obj->motion = ii_goal_open1;
			break;
		case 3:
			obj->motion = ii_goal_closed;
            SetState(obj, os_none);
			break;
		default:
			assert(0);
			break;
		}
	default:
		break;
	}
	return 0;
}

int act_fwall(obj_t *obj){
	switch (State(obj)){
    case os_none:
	    if (check_emap_obj_any_rim(obj, EM_SHOCK)){
			// ɤblockingemap
			reset_emap_static_blockbox(obj->x, obj->y, obj->width, obj->height);
			reset_emap_static_box(obj->x, obj->y, obj->width, obj->height,
                                  EM_FWALL);
			SetState(obj, os_bang);
			obj->motion = ii_bang4_4;
			inc_kill_record(obj->type);
		}
		break;
    case os_bang:
		IncCtr(obj);
		if (Ctr(obj) == BANG_SPEED){
			return 1; // forall_obj¦ǺƤ餦
		}
		break;
	default:
		assert(0);
		break;
    }
    return 0;
}
int act_iwall(obj_t *obj){
	if (check_emap_obj_any_rim(obj, EM_FLAME)){
		IncCtr(obj);
		if(Ctr(obj) >= IWALL_LIFE){
			// ɤblockingemap
			reset_emap_static_blockbox(obj->x, obj->y, obj->width, obj->height);
			reset_emap_static_box(obj->x, obj->y, obj->width, obj->height,
                                  EM_ICE);
			inc_kill_record(obj->type);
			return 1;
		}
	}
	return 0;
}
int act_pipe(obj_t *obj){
	int interval;
	switch (State(obj)){
    case os_req_off:
        SetState(obj, os_off);
        break;
    case os_req_on:
        SetState(obj, os_on);
        break;
    case os_on:
        IncCtr(obj);
        switch (obj->dir){
        case di_left:
            interval = DROP_LONG_INTERVAL;
            break;
        case di_up:
            interval = DROP_INTERVAL;
            break;
        case di_right:
            interval = DROP_SHORT_INTERVAL;
            break;
        case di_down:
            interval = 0;
            break;
        default:
            assert(0);
            break;
        }
        //		interval = (obj->dir == di_left) ? DROP_LONG_INTERVAL :
        //					(obj->dir == di_right) ? DROP_SHORT_INTERVAL : DROP_INTERVAL;
        if (Ctr(obj) >= interval){
            // ®ٻ꤬vʤ顢ũȤ
            // @note ֥åȽOFFˤƤʤȡ
            // ֥å׵᤬֥åľȯưƤޤ
			if (obj->dir == di_down){
				SetState(obj, os_off);
			}
			else{
	            SetCtr(obj, 0);
			}
			if (check_emap_dir(EM_BLOCK_DOWN, di_down, obj)) break;
//			if (check_emap_box_any(Ltx(obj), Rby(obj), obj->width, 1, 
            create_obj(ot_siguku, obj->x, obj->y+1, di_none);
        }
        break;
    case os_off:
        break;
    default:
        break;
    }
	return 0;
}
int act_barrier(obj_t *obj){
    switch (State(obj)){
    case os_req_off:
        //        puts("BARRIER REQ OFF");
        SetState(obj, os_off);
        //        obj->motion = ii_none;
        break;
        /*
    case os_off:
        //        puts("BARRIER OFF");
        break;
        */
    case os_req_on:
        //        puts("BARRIER REQ_ON");
        if (PrevState(obj) != os_on){
            printf("req barrier on:(%d, %d) %dx%d\n", obj->x, obj->y, obj->width, obj->height);
            if (!check_emap_box_any_rim(Ltx(obj)+1, Lty(obj)+1,
                                        obj->width-2, obj->height-2,
                                        EM_BLOCK_LEFT) &&
                !check_emap_box_any_rim(Ltx(obj)+1, Lty(obj)+1,
                                        obj->width-2, obj->height-2, 
                                        EM_BLOCK_RIGHT) &&
                !check_emap_box_any_rim(Ltx(obj)+1, Lty(obj)+1,
                                        obj->width-2, obj->height-2,
                                        EM_BLOCK_UP) &&
                !check_emap_box_any_rim(Ltx(obj)+1, Lty(obj)+1,
                                        obj->width-2, obj->height-2,
                                        EM_BLOCK_DOWN))
            {
                SetState(obj, os_on);
                obj->motion = ii_barrier0;
            }
            break;
        }
        else {
            // ONФreq_onä̵뤹
            // ʣϤƱڤؤäȤˡ
            // Ʊ쥿ON/OFFȯΤɤ
            SetState(obj, os_on);
            //            /* FALLTHRU */
            break;
        }
        /*
    case os_on:
        //        puts("BARRIER ON");
		if (obj->motion == ii_barrier3){
			obj->motion = ii_barrier0;
		}
		else {
			obj->motion ++;
		}
        break;
        */
    default:
        break;
    }
    set_motion_barrier(obj);
    set_emap_barrier(obj);
    SavePrevState(obj);
    return 0;
}

int setup_barrier(obj_t *obj){
    act_barrier(obj);
    set_emap_barrier(obj);
    return 0;
}

void set_motion_barrier(obj_t *obj){
    if (State(obj) == os_on){
		if (GetMotion(obj) == ii_barrier3){
			SetMotion(obj, ii_barrier0);
		}
		else {
			obj->motion ++;
		}
    }
    else {
        SetMotion(obj, ii_none);
    }
}

void set_emap_barrier(obj_t *obj){
    if (State(obj) == os_on){
		// ʤɤ­եåȤΤξ¦ǻ˴ȽˤʤäƤޤ
		// ˤƤ
        set_emap_box(obj->x+1, obj->y+1, obj->width-2, obj->height-2, EM_ELECTRIC);
        // Хꥢȯ꤮˿եȤʤɤΤޤ̲᤻
        // äǤȿž褦blockʣ饤ĥ
        // !!! blockboxʣƤꤹΤ̵̤ʤΤ
        // ȼemapꥳɤ񤯤褦˽
        set_emap_blockbox(obj->x, obj->y, obj->width, obj->height);
        set_emap_blockbox(obj->x+1, obj->y+1, obj->width-2, obj->height-2);
        set_emap_blockbox(obj->x+2, obj->y+2, obj->width-4, obj->height-4);
    }
}

int act_flame(obj_t *obj){
    switch(State(obj)){
    case os_teleporting:
        teleport_creature(obj);
        return 0;
    default:
        break;
    }
    if (start_teleporting_creature(obj)){
        return 0;
    }
    IncCtr(obj);
	if (Ctr(obj) == FLAME_COUNT) return 1;
    if (check_emap_box_any_rim(Ltx(obj), Lty(obj), FLAME_WIDTH, FLAME_HEIGHT,
                               EM_EXCLUDE_LEFT | EM_EXCLUDE_RIGHT | EM_WATER))
    {
        return 1;
    }
    obj->x += (obj->dir == di_left) ? -FLAME_WIDTH+1 : FLAME_WIDTH-1;
    set_emap_flame(obj);
	return 0;
}


void set_emap_flame(obj_t *obj){
    set_emap_box(obj->x, obj->y, obj->width, obj->height, EM_FLAME);
}


