#include <stdio.h>
#include <EzGraph.h>

#include "config.h"
#include "layout.h"
#include "draw.h"
#include "select_stage.h"
#include "select_stage_draw.h"
#include "obj_draw.h"
#include "font.h"

#define TLtx(x) ((x)*THUMB_XSKIP + THUMB_XOFFSET)
#define TLty(y) ((y)*THUMB_YSKIP + THUMB_YOFFSET)

static char *title_str = "ơ󤿤";
static font_id_t *title_fontseq;
static char *star_unit_str = "[StarOn]";
static font_id_t *star_unit_fontseq;
static char *stage_unit_str = "";
static font_id_t *stage_unit_fontseq;
static char *score_unit_str = "Ƥ";
static font_id_t *score_unit_fontseq;

static char *stage_clear_str = "ꤢ";
static font_id_t *stage_clear_fontseq;

void select_stage_draw_init(void){
    title_fontseq = alloc_fontseq(title_str);
    stage_unit_fontseq = alloc_fontseq(star_unit_str);
    stage_unit_fontseq = alloc_fontseq(stage_unit_str);
    score_unit_fontseq = alloc_fontseq(score_unit_str);
    stage_clear_fontseq = alloc_fontseq(stage_clear_str);
    star_unit_fontseq = alloc_fontseq(star_unit_str);
}

void draw_select_stage(stage_thumb_t stage_thumbs[], int stage_num,
                       int cursor_x, int cursor_y, int line)
{
    int x, y, si, i;
    font_id_t star_num_fontseq[STAR_NUM_LEN+1+STAR_NUM_LEN+1];
    font_id_t no_fontseq[STAGE_NUM_LEN+1];
    font_id_t score_fontseq[SCORE_LEN+1];
    font_id_t stage_num_fontseq[STAGE_NUM_LEN+1+STAGE_NUM_LEN+1];

    uint2fontseq(line*THUMB_COL_NUM, STAGE_NUM_LEN, no_fontseq);
    draw_fontseq((WIN_WIDTH-16*9)/2, 8, title_fontseq);


    uint2fontseq(get_achieved_star_num(), STAR_NUM_LEN, 
                 star_num_fontseq);
    star_num_fontseq[STAR_NUM_LEN] = fi_slash;
    uint2fontseq(get_total_star_num(), STAR_NUM_LEN,
                 star_num_fontseq + STAR_NUM_LEN+1);
    draw_fontseq(WIN_WIDTH/2 - 16*(STAR_NUM_LEN+1), 32, star_unit_fontseq);
    draw_fontseq(WIN_WIDTH/2 - 16*(STAR_NUM_LEN), 32, star_num_fontseq);

    uint2fontseq(get_cleared_stage_num(), STAGE_NUM_LEN, 
                 stage_num_fontseq);
    stage_num_fontseq[STAGE_NUM_LEN] = fi_slash;
    uint2fontseq(get_stage_num(), STAGE_NUM_LEN,
                 stage_num_fontseq + STAGE_NUM_LEN+1);
    draw_fontseq(16*4, 32, stage_num_fontseq);
    draw_fontseq(16*(4+STAGE_NUM_LEN*2+1), 32, stage_clear_fontseq);
    /*
    draw_fontseq(WIN_WIDTH/2 - 16*(3+2+STAGE_NUM_LEN*2+1), 32,
                 stage_num_fontseq);
    draw_fontseq(WIN_WIDTH/2 - 16*(3+2), 32, stage_clear_fontseq);
    */

    uint2fontseq(get_total_score(), SCORE_LEN, score_fontseq);
    draw_fontseq(WIN_WIDTH/2 + 16*8, 32, score_fontseq);
    draw_fontseq(WIN_WIDTH/2 + 16*(8 + SCORE_LEN), 32, score_unit_fontseq);
    /*
    draw_fontseq(WIN_WIDTH/2 + 16*2, 32, score_fontseq);
    draw_fontseq(WIN_WIDTH/2 + 16*(2 + SCORE_LEN), 32, score_unit_fontseq);
    */
    for (y = 0 ; y < THUMB_ROW_NUM ; y++){
        for (x = 0 ; x < THUMB_COL_NUM ; x++){
            si = CursorPos2StageIx(x, y, line);
            if (si < stage_num){
                SetColor(7);


                inc_fontseq(STAGE_NUM_LEN, no_fontseq);
                draw_fontseq(TLtx(x)+THUMB_WIDTH-16*(STAGE_NUM_LEN), TLty(y)-32,
                             no_fontseq);
#ifdef LOCK_STAGE
                if (!stage_thumbs[si].unlocked){
                    draw_stage_unknown(TLtx(x), TLty(y));
                    continue;
                }
#endif
                draw_fontseq(TLtx(x), TLty(y)-32,
			  				stage_thumbs[si].stars_fontseq);
				uint2fontseq(stage_thumbs[si].top_score, SCORE_LEN, score_fontseq);
//                draw_fontseq(TLtx(x)+THUMB_WIDTH-16*(STAGE_NUM_LEN+2), TLty(y)-32,
//                             no_fontseq);
//                draw_fontseq(TLtx(x)+THUMB_WIDTH-16*2, TLty(y)-32,
//                             stage_unit_fontseq);
                draw_fontseq(TLtx(x)+THUMB_WIDTH-16*(SCORE_LEN+2), TLty(y)-16,
                             score_fontseq);
                draw_fontseq(TLtx(x)+THUMB_WIDTH-16*2, TLty(y)-16,
                             score_unit_fontseq);
                draw_stage_thumb(&stage_thumbs[si], TLtx(x), TLty(y));
                for (i = 0 ; i < STAGE_TITLE_ROW_NUM ; i++){
                    draw_fontseq(TLtx(x), TLty(y)+THUMB_HEIGHT+i*LINESKIP+8,
                                 stage_thumbs[si].title[i]);
                }
#ifdef DEBUG
                //                EzDrawStringB(TLtx(x), TLty(y)+64, get_stage_filename(si));
#endif
            }
        }
    }
    draw_img(ii_chee_back, 
             RDX(TLtx(cursor_x)+THUMB_WIDTH/2-16),
             RDY(TLty(cursor_y)+THUMB_HEIGHT/2-16));
    EzShowBuffer();
}

void draw_stage_thumb(stage_thumb_t *thumb, int ltx, int lty){
	int x, y;
    for (y = 0 ; y < MAP_GRID_HEIGHT ; y++){
        for (x = 0 ; x < MAP_GRID_WIDTH ; x++){
			SetColor(GetThumbColor(x, y));
			EzFillBoxB(x*4+ltx, y*4+lty, x*4+3+ltx, y*4+3+lty);
		}
	}
}

#ifdef LOCK_STAGE
void draw_stage_unknown(int ltx, int lty){
    SetColor(2);
    EzFillBoxB(ltx, lty, ltx+MAP_GRID_WIDTH*4-1, lty+MAP_GRID_HEIGHT*4-1);
    SetColor(0);
    EzFillBoxB(ltx+4, lty+4, ltx+MAP_GRID_WIDTH*4-5, lty+MAP_GRID_HEIGHT*4-5);
    draw_font(ltx+(MAP_GRID_WIDTH*4/2)-8, lty+(MAP_GRID_HEIGHT*4/2)-8, 
              fi_question);
}
#endif
